$TITLE ('GALEX/COS Command Handling')
$PAGELENGTH(59)
$SB
$DEBUG
;THIS IS COMMANDS.A51,                        LAST EDITED ::  7-23-100 1:20 am 
;*****************************************************************************
;
;   CCCCC
;  C     C    OOOO    M    M   M    M     AA     N    N   DDDDD     SSSS
;  C         O    O   MM  MM   MM  MM    A  A    NN   N   D    D   S
;  C         O    O   M MM M   M MM M   A    A   N N  N   D    D    SSSS
;  C         O    O   M    M   M    M   AAAAAA   N  N N   D    D        S
;  C     C   O    O   M    M   M    M   A    A   N   NN   D    D   S    S
;   CCCCC     OOOO    M    M   M    M   A    A   N    N   DDDDD     SSSS
;
;*****************************************************************************
; v16242 dbb 11-25-98 Added bits for HV ramping
; v16257 dbb 12-24-98 Updated diagnostic codes, added AUXI overcurrent (2E)
;                     Changed TEST D to select ramp from 0, 15, 3, 1
; v16258 dbb 12-24-98 Fixed incorrect diag 2B-->2D
; v16262 dbb 01-01-99 Moved CLR fFORMAT to initialz on reset only.
; v16300 dbb 01-08-99 Released to FUSE project for upload to v16200 in PROM.
; v16402 dbb 01-08-99 no serial out during HV
; v16404 dbb 01-09-99 no serial out after overlimit trigger
; v16405 dbb 02-05-99 SAFE_CONTROLS bits reset properly...
;                     nVERBOSE added to REPORT_DIAGNOSTIC, init on
;                     HVIB overlimit diag fixed 27->28
;                     Updated diagnostic text messages
;                     ASSERTED to suppress diag 22,23 in REDUCE/RESUME
; v16408 dbb 02-06-99 nVERBOSE set on Reentry, Test E; reset on Test D
;                     Test D used to select Ramping
; v16412 dbb 02-06-99 SAA HV Override command will reset SAA machine
; v16413 dbb 02-09-99 Dropped FSTAT in favor of timestamping records with CL
; v16414 dbb 02-09-99 Zero lower Code CRC on jmpcode to prevent diagnostic
; v16415 dbb 02-09-99 Test D toggling treats 0 like any other ramp value
; v16419 dbb 02-09-99 Use SPARE_Sx_CMD commands to work around eGSE limitation
; v16420 dbb 02-09-99 Ramp is 0 by default
; v16420 dbb 02-10-99 Clean up for delivery as v165
; v16500 dbb 02-12-99 Upload version released.
; v16732 dbb 12-02-99 Repaired error in HARD_RESET routine
; v16741 dbb 12-06-99 Repaired tLFPINTA
; v16743 dbb 12-13-99 Added HKPFULL and hkp changes only
; v16778 dbb 02-10-00 Filled unused space with jumps to trap
; v16791 dbb 02-29-00 Updated and tested DUMP command
; v16796 dbb 03-02-00 Developing a new copy method using a pointer to table
; v16819 dbb 03-14-00 Using LFDPEEK = pointmon to return debug info on command
; v16839 dbb 03-29-00 C1_LOCK now set to locked state after init
; v16845 dbb 04-03-00 Rewrote CDC commanding scheme to support differing
;                     layouts in Digitizer/TDC.  Pursuant to design goals,
;                     the TDC layout is specified in a table of commands
;                     that pairs equilvalent GALEX/COS Digitizer/TDC commands.
; v16862 dbb 04-23-00 For GALEX, HV_MAX must be implemented in software
; v16867 dbb 04-28-00 Special test command fcfc and partial Breadboard version.
; v16878 dbb 05-04-00 Stim commands developed
; v16882 dbb 05-05-00 Added side B TDC Sampling and Control
;                     Commands with special parameters are not yet implemented
; v16885 dbb 05-07-00 REORGANIZED! New memory and IVT layout, future
;                     uploads will not be compatable with older versions.
; v17000 dbb 05-22-00 Code Review Version
; v17025 dbb 06-13-00 Can't readback CCAPM4, so WATCHDOG_STATE must be updated
;                     whenever the Watchdog is initialized or disabled.
; v17034 dbb 06-13-00 Fixed HST errors so it works by reporting the value of
;                     the word pointed-to by the item in the HSTERR_TABLE
;                     that corresponds to a diagnostic code.
;*****************************************************************************
$SET (JNTCODE)
$SET (COMMANDS)
$INCLUDE(common.inc)

         NAME COMMANDS


;*****************************************************************************
COMMANDS SEGMENT CODE PAGE
           RSEG COMMANDS
$EJECT
;*****************************************************************************
;
; Subroutine  HARD_RESET
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HARD_RESET:
    MOV    SDBUFFR+2,#11h     ; cause a power-up boot sequence
SOFT_RESET:
    CPL    HARDRESET          ; toggle reset line to trip POR circuit
    TOGLEDS 55h
    AJMP   SOFT_RESET

; We could get here if the reset line is not connected, as in breadboard.
    REPORT_DIAG DIAG0001      ; anything... dbb 9-27-9
    CPL    HARDRESET          ; toggle reset line -- this might do it!
    JMP    0                  ; jmp to reset vector

    SPARE_CODE   8            ; insert nops to align the next section
$EJECT
;*****************************************************************************
;
; Subroutine  OPERATEBIT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
OPERATEBIT:
    PUSHDPTR
    PUSH   ACC
    MOV    DPTR,#OPERATEBIT   ; get a nearby address
    MOV    A,DPH              ; get the high byte
    ANL    A,#80h             ; look at MSBit
    CLR    OPERATES           ; assume Boot Mode
    JZ     notOperate         ; check if bit is set
    SETB   OPERATES           ; then Operate Mode
notOperate:
    POP    ACC
    POPDPTR
    RET                       ; OPERATEBIT
$EJECT
;*****************************************************************************
;
; Subroutine  COMMAND_VECTOR
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v17048 dbb 07-09-00 COUNT_CMDS_RXD must be called with A masked command
;
; Action Item 1.021
;     Jump Table format is completely redesigned, but not quite the way Will
; had specified.  JMPs were competely eliminated to create a lookup table of
; command subroutine entry-points.  The CHECK_COMMAND routine in commands.a51
; was changed to fetch the subroutine address and jump there.  The PROJECT bit
; is used to select the command table, allowing separate command tables for
; GALEX and COS.  The RET instruction at the end of every command subroutine
; returns control to the MAINLOOP caller.
;
;*****************************************************************************
COMMAND_VECTOR:
    CALL   OPERATEBIT         ; update Boot/Operate Mode flag
    ANL    A,#7Fh             ; Ignore high-order bit, A holds command byte
    CALL   COUNT_CMDS_RXD     ; increment the commands received counter
    JB     PROJECT,CV0        ; COS requires commands not specified in the
                              ; ICD to generate a diagnostic when invoked,
                              ; so we have to have separate command tables
                              ; for GALEX and COS.
    MOV    DPTR,#GAL_JMP_TBL  ; Use jump table below to select command
    JMP    CV1		      ;  GALEX commands selected
CV0:
    MOV    DPTR,#COS_JMP_TBL  ; Use jump table below to select command
CV1:                          ; else COS commands selected.
    CALL   FETCHPTR
    CLR    A                  ; dptr holds the command function pointer, offset A is zero
    JMP    @A+DPTR
COS_JMP_TBL:
    DW     NO_OPER            ; 00H *No Operation, can be used as  Ping to check aliveness
    DW     PROGRAMMONITOR     ; 01H  index,addr,bank
    DW     CALC_CRC           ; 02H  Inserts specific CRC region in background
    DW     MEM_COPY           ; 03H  Copy memory-to-memory on command
    DW     NO_OPER            ; 04H  No Operation, undefined command
    DW     BEG_WLK_A          ; 05H *SET BITS FOR TDC-A START (BEGIN) CFD WALK ADJUSTMENT
    DW     GDGBWKY            ; 06H *SET BITS FOR TDC-B START (BEGIN) CFD WALK ADJUSTMENT
    DW     NO_OPER            ; 07H  SET BITS FOR TDC-A CHARGE BASELINE INSPECTOR WINDOW
    DW     NO_OPER            ; 08H  SET BITS FOR TDC-B CHARGE BASELINE INSPECTOR WINDOW
    DW     END_WLK_A          ; 09H *SET BITS FOR TDC-A STOP (CFD) WALK ADJUSTMENT
    DW     GDGEWKY            ; 0AH *SET BITS FOR TDC-B STOP (CFD) WALK ADJUSTMENT
    DW     NO_OPER            ; 0BH  No Operation, undefined command
    DW     NO_OPER            ; 0CH  No Operation, undefined command
    DW     HV_A_MAX           ; 0DH *set maximum setting for HV A (via DAC and clamping diode)
    DW     HV_B_MAX           ; 0EH  set maximum setting for HV B (via DAC and clamping diode)
    DW     NO_OPER            ; 0FH  Mask Box image operation (fast)
    DW     NO_OPER            ; 10H  Mask Box image operation (fast)
    DW     NO_OPER            ; 11H  Mask Box image operation (fast)
    DW     NO_OPER            ; 12H  Mask Box image operation (fast)
    DW     NO_OPER            ; 13H  Mask Box image operation (fast)
    DW     NO_OPER            ; 14H  Mask Box image operation (fast)
    DW     LWR_Q_THR_A        ; 15H *TDC-A LOWER CHARGE (Y) THRESHOLD
    DW     LWR_Q_THR_B        ; 16H  SET BITS FOR TDC-B LOWER CHARGE (Y) THRESHOLD
    DW     LWR_T_THR_A        ; 17H *TDC-A LOWER TIME (X) THRESHOLD GDGTTX
    DW     GDGTTY             ; 18H *TDC-B LOWER TIME (Y) THRESHOLD
    DW     NO_OPER            ; 19H  No Operation, undefined command
    DW     NO_OPER            ; 1AH  No Operation, undefined command
    DW     NO_OPER            ; 1BH  No Operation, undefined command
    DW     MOTOR_SAFETY_OVERRIDE ; 1Ch Overrides Endswitches
    DW     MODE               ; 1DH  TURNS ON_OFF STIM LAMP
    DW     LFGSTIM            ; 1Eh  ; (STIM_ON_OFF_A) TDC-A STIMULUS CONTROL
    DW     MODE               ; 1FH  TURNS TDC-B STIMULUS GENERATOR ON_OFF
    DW     MODE               ; 20H  SETS STIMULUS GENERATOR A FOR SWEEPING AMPLITUDES
    DW     MODE               ; 21H  SETS STIMULUS GENERATOR B FOR SWEEPING AMPLITUDES
    DW     UPR_Q_THR_A        ; 22H *SET BITS FOR TDC-A UPPER CHARGE (Y) THRESHOLD
    DW     UPR_Q_THR_B        ; 23H  SET BITS FOR TDC-B UPPER CHARGE (Y) THRESHOLD
    DW     X_OFFSET_A         ; 24H  SET BITS FOR TDC-A TIME (X) IMAGE POSITION OFFSET
    DW     X_OFFSET_B         ; 25H  SET BITS FOR TDC-B TIME (X) IMAGE POSITION OFFSET
    DW     X_SCALE_A          ; 26H  SET BITS FOR TDC-A TIME (X) SCALE FACTOR
    DW     MODE               ; 27H  SET BITS FOR TDC-B TIME (X) SCALE FACTOR
    DW     MODE               ; 28H  set observation mode (observing, special ops, safe, ..
    DW     cFORMAT            ; 29H  set data format on RS-422{  (for calibration)}
    DW     NO_OPER            ; 2AH  No Operation, undefined command
    DW     TEST               ; 2BH *Enter test mode. (dump pseudo telemetry ...)
    DW     CHECK              ; 2CH *Check a value (returned as a monitor)
    DW     UPLOAD             ; 2DH  Loads memory with Upload Data block: Startaddr, Endadd
    DW     DUMP               ; 2EH *Dumps memory block: Startaddr, Length, DataCRC32
    DW     SPARE_O1_CMD       ; 2FH   spare
    DW     NO_OPER            ; 30H  LOADS IMAGE MASKS FOR SEGMENT A, Start, End, DataCRC32
    DW     NO_OPER            ; 31H  LOADS IMAGE MASKS FOR SEGMENT B, Start, End, DataCRC32
    DW     NO_OPER            ; 32H  Dumps mask A: Startaddr, Endaddr, DataCRC32
    DW     NO_OPER            ; 33H  Dumps mask B: Startaddr, Endaddr, DataCRC32
    DW     SAA_CLMT_A         ; 34H *SAA COUNT LIMIT FOR SEGMENT A AUTO HV REDUCE
    DW     SAA_CLMT_B         ; 35H  SAA COUNT LIMIT FOR SEGMENT B AUTO HV REDUCE
    DW     SAA_HV_A           ; 36H *SAA HV REDUCED VALUE SEGMENT A
    DW     SAA_HV_B           ; 37H  SAA HV REDUCED VALUE SEGMENT B
    DW     SAA_HV_OVRD_A      ; 38H  SAA HV AUTO SHUTDOWN OVER-RIDE SEGMENT A
    DW     SAA_HV_OVRD_B      ; 39H  SAA HV AUTO SHUTDOWN OVER-RIDE SEGMENT B
    DW     SAA_INT_A          ; 3AH *SAA INTEGRATION TIME FOR SEGMENT A AUTO HV REDUCE
    DW     SAA_INT_B          ; 3BH  SAA INTEGRATION TIME FOR SEGMENT B AUTO HV REDUCE
    DW     SAA_TOFF           ; 3CH *SAA SEGMENT AUTO HV REDUCE OFF PERIOD
    DW     SPARE_S0_CMD       ; 3DH   spare
    DW     SPARE_S1_CMD       ; 3EH   spare
    DW     SPARE_S2_CMD       ; 3FH   spare
    DW     HV_ON              ; 40H *switches HV BIAS supplies on after writing 0 to the DA
    DW     ACT_1_ON           ; 41H  TURNS ON_OFF ACTUATOR 1
    DW     ACT_2_ON           ; 42H  TURNS ON_OFF ACTUATOR 2
    DW     cACT_ENA           ; 43H  ARMS 28V ACTUATOR PWR SWITCH
    DW     cAUX_PWR_ENA       ; 44H  Turns ON_OFF 28V AUX OSCILLATOR for Motor, Actuator, I
    DW     ACT_RESET          ; 45H  Actuator reset sequence
    DW     HVNOMINAL          ; 46H *LFHVNMA Nom value for HV A
    DW     HVNOMINBL          ; 47H  LFHVNMB Nom value for HV B
    DW     NO_OPER            ; 48H  No Operation, undefined command
    DW     NO_OPER            ; 49H  No Operation, undefined command
    DW     MODE               ; 4AH *CRP_OVR CRP override
    DW     HV_A_CMD           ; 4BH *SET BITS FOR MCP-A HV BIAS
    DW     HV_B_CMD           ; 4CH  SET BITS FOR MCP-B HV BIAS
    DW     NO_OPER            ; 4DH  No Operation, undefined command
    DW     RAMPTIMEC          ; 4EH *HV_RAMP HV Ramp time constant
    DW     HVQ_ON             ; 4FH  switches MCP grid supply on
    DW     HV_I_MAX           ; 50H *Set HV current limit
    DW     HV_ARM             ; 51H *Enables HV_ON and HVQ_ON (in the DPU = software)
    DW     MODE               ; 52H  TURNS ON ION PUMP #1
    DW     MODE               ; 53H  TURNS ON ION PUMP #2
    DW     MODE               ; 54H  ENABLES ION PUMPS #1& #2
    DW     GDGBOFFY           ; 55H *TDC Begin Y interpolator offset Y_SHIFT_DAC
    DW     GDGEOFFY           ; 56H *TDC End Y interpolator  offset Y_SHIFT_DAE
    DW     GDGBSCLY           ; 57H *TDC Begin Y interpolator scale Y_SCALE_DAC
    DW     GDGESCLY           ; 58H *TDC End Y interpolator scale Y_SCALE_DAE
    DW     MOTOR_CNTRL        ; 59H  Sets motor bridge switches (direction) avoid M1=M2=1
    DW     MOTOR_ENA          ; 5AH  ENABLES MOTOR BRIDGE SWITCHES (may start the motor)
    DW     MOTOR_ARM          ; 5BH  Allows MOTOR_ENA (in the DPU)
    DW     DOOR_I_MAX         ; 5CH  Set door current limit
    DW     NO_OPER            ; 5DH  No Operation, undefined command
    DW     NO_OPER            ; 5EH  No Operation, undefined command
    DW     NO_OPER            ; 5FH  No Operation, undefined command
    DW     MODE               ; 60H  Mask Box image operation
    DW     GDGBOFFX           ; 61H *TDC Begin X interpolator  offset [#DGBOFFX]
    DW     GDGEOFFX           ; 62H *TDC End X interpolator  offset [#DGEOFFX]
    DW     GDGBSCLX           ; 63H *TDC Begin X interpolator scale [#DGBSCLX]
    DW     GDGESCLX           ; 64H *TDC End X interpolator scale [#DGESCLX]
    DW     MODE               ; 65H  Mask Box image operation
    DW     HSTATE             ; 66H *Trigger shutdown, no auto restore
    DW     NOMSTATE           ; 67H *Restore HV saved during shutdown
    DW     MODE               ; 68H  Set SAA shutdown values to 80% of HV
    DW     END_COMMAND        ; 69H  Don't expect anymore commands, no reset
    DW     GOTO               ; 6AH *Jump to specified address and execute code there
    DW     CRPMODE            ; 6BH *CRP enable/disable
    DW     NO_OPER            ; 6CH  No Operation, undefined command
    DW     NO_OPER            ; 6DH  No Operation, undefined command
    DW     SAA_OVER           ; 6EH *day - Restore HV saved during shutdown
    DW     SAA_HERE           ; 6FH *night - Trigger shutdown, no auto restore
    DW     HARD_RESET         ; 70H *power-on reset
    DW     WD_STATE           ; 71H *enable/disable watchdog
    DW     NO_OPER            ; 72H  No Operation, undefined command
    DW     JMPCODE            ; 73H *jump code
    DW     ZAP_TS30           ; 74H *clear diags
    DW     SOFT_RESET         ; 75H *watchdog reset
    DW     SAFE_CONTROLS      ; 76H *Enter Idle state, HV off [#DSBITS5]
    DW     DEBUGG             ; 77H  No Operation, undefined command
    DW     NO_OPER            ; 78H  No Operation, undefined command
    DW     NO_OPER            ; 79H  No Operation, undefined command
    DW     ACTEL_RESET        ; 7AH  Toggle reset line to counter actel
    DW     TDC                ; 7BH  TDC debug dbb 4-14-00
    DW     JNTTEST            ; 7CH  Reserved for J&T
    DW     PROM_STATE         ; 7DH *Set PROM bit according to parameter
    DW     TDC_A_TEST         ; 7EH  dbb 4-1-00
    DW     HKPFULL            ; 7FH *Send full copy of Housekeeping Memory
GAL_JMP_TBL:
    DW     NO_OPER            ; 00H *No Operation, can be used as  Ping to check aliveness
    DW     PROGRAMMONITOR     ; 01H  index,addr,bank
    DW     CALC_CRC           ; 02H  Inserts specific CRC region in background
    DW     MEM_COPY           ; 03H  Copy memory-to-memory on command
    DW     NO_OPER            ; 04H  No Operation, undefined command
    DW     BEG_WLK_A          ; 05H *SET BITS FOR TDC-A START (BEGIN) CFD WALK ADJUSTMENT
    DW     GDGBWKY            ; 06H *SET BITS FOR TDC-B START (BEGIN) CFD WALK ADJUSTMENT
    DW     NO_OPER            ; 07H  SET BITS FOR TDC-A CHARGE BASELINE INSPECTOR WINDOW
    DW     NO_OPER            ; 08H  SET BITS FOR TDC-B CHARGE BASELINE INSPECTOR WINDOW
    DW     END_WLK_A          ; 09H *SET BITS FOR TDC-A STOP (CFD) WALK ADJUSTMENT
    DW     GDGEWKY            ; 0AH *SET BITS FOR TDC-B STOP (CFD) WALK ADJUSTMENT
    DW     NO_OPER            ; 0BH  No Operation, undefined command
    DW     NO_OPER            ; 0CH  No Operation, undefined command
    DW     HV_A_MAX           ; 0DH *set maximum setting for HV A (via DAC and clamping diode)
    DW     HV_B_MAX           ; 0EH  set maximum setting for HV B (via DAC and clamping diode)
    DW     NO_OPER            ; 0FH  Mask Box image operation (fast)
    DW     NO_OPER            ; 10H  Mask Box image operation (fast)
    DW     NO_OPER            ; 11H  Mask Box image operation (fast)
    DW     NO_OPER            ; 12H  Mask Box image operation (fast)
    DW     NO_OPER            ; 13H  Mask Box image operation (fast)
    DW     NO_OPER            ; 14H  Mask Box image operation (fast)
    DW     LWR_Q_THR_A        ; 15H *TDC-A LOWER CHARGE (Y) THRESHOLD
    DW     LWR_Q_THR_B        ; 16H  SET BITS FOR TDC-B LOWER CHARGE (Y) THRESHOLD
    DW     LWR_T_THR_A        ; 17H *TDC-A LOWER TIME (X) THRESHOLD GDGTTX
    DW     GDGTTY             ; 18H *TDC-B LOWER TIME (Y) THRESHOLD
    DW     NO_OPER            ; 19H  No Operation, undefined command
    DW     NO_OPER            ; 1AH  No Operation, undefined command
    DW     NO_OPER            ; 1BH  No Operation, undefined command
    DW     MOTOR_SAFETY_OVERRIDE ; 1Ch Overrides Endswitches
    DW     MODE               ; 1DH  TURNS ON_OFF STIM LAMP
    DW     LFGSTIM            ; 1Eh  ; (STIM_ON_OFF_A) TDC-A STIMULUS CONTROL
    DW     MODE               ; 1FH  TURNS TDC-B STIMULUS GENERATOR ON_OFF
    DW     MODE               ; 20H  SETS STIMULUS GENERATOR A FOR SWEEPING AMPLITUDES
    DW     MODE               ; 21H  SETS STIMULUS GENERATOR B FOR SWEEPING AMPLITUDES
    DW     UPR_Q_THR_A        ; 22H *SET BITS FOR TDC-A UPPER CHARGE (Y) THRESHOLD
    DW     UPR_Q_THR_B        ; 23H  SET BITS FOR TDC-B UPPER CHARGE (Y) THRESHOLD
    DW     X_OFFSET_A         ; 24H  SET BITS FOR TDC-A TIME (X) IMAGE POSITION OFFSET
    DW     X_OFFSET_B         ; 25H  SET BITS FOR TDC-B TIME (X) IMAGE POSITION OFFSET
    DW     X_SCALE_A          ; 26H  SET BITS FOR TDC-A TIME (X) SCALE FACTOR
    DW     MODE               ; 27H  SET BITS FOR TDC-B TIME (X) SCALE FACTOR
    DW     MODE               ; 28H  set observation mode (observing, special ops, safe, ..
    DW     cFORMAT            ; 29H  set data format on RS-422{  (for calibration)}
    DW     NO_OPER            ; 2AH  No Operation, undefined command
    DW     TEST               ; 2BH *Enter test mode. (dump pseudo telemetry ...)
    DW     CHECK              ; 2CH *Check a value (returned as a monitor)
    DW     UPLOAD             ; 2DH  Loads memory with Upload Data block: Startaddr, Endadd
    DW     DUMP               ; 2EH *Dumps memory block: Startaddr, Length, DataCRC32
    DW     SPARE_O1_CMD       ; 2FH   spare
    DW     NO_OPER            ; 30H  LOADS IMAGE MASKS FOR SEGMENT A, Start, End, DataCRC32
    DW     NO_OPER            ; 31H  LOADS IMAGE MASKS FOR SEGMENT B, Start, End, DataCRC32
    DW     NO_OPER            ; 32H  Dumps mask A: Startaddr, Endaddr, DataCRC32
    DW     NO_OPER            ; 33H  Dumps mask B: Startaddr, Endaddr, DataCRC32
    DW     SAA_CLMT_A         ; 34H *SAA COUNT LIMIT FOR SEGMENT A AUTO HV REDUCE
    DW     SAA_CLMT_B         ; 35H  SAA COUNT LIMIT FOR SEGMENT B AUTO HV REDUCE
    DW     SAA_HV_A           ; 36H *SAA HV REDUCED VALUE SEGMENT A
    DW     SAA_HV_B           ; 37H  SAA HV REDUCED VALUE SEGMENT B
    DW     SAA_HV_OVRD_A      ; 38H  SAA HV AUTO SHUTDOWN OVER-RIDE SEGMENT A
    DW     SAA_HV_OVRD_B      ; 39H  SAA HV AUTO SHUTDOWN OVER-RIDE SEGMENT B
    DW     SAA_INT_A          ; 3AH *SAA INTEGRATION TIME FOR SEGMENT A AUTO HV REDUCE
    DW     SAA_INT_B          ; 3BH  SAA INTEGRATION TIME FOR SEGMENT B AUTO HV REDUCE
    DW     SAA_TOFF           ; 3CH *SAA SEGMENT AUTO HV REDUCE OFF PERIOD
    DW     SPARE_S0_CMD       ; 3DH   spare
    DW     SPARE_S1_CMD       ; 3EH   spare
    DW     SPARE_S2_CMD       ; 3FH   spare
    DW     HV_ON              ; 40H *switches HV BIAS supplies on after writing 0 to the DA
    DW     ACT_1_ON           ; 41H  TURNS ON_OFF ACTUATOR 1
    DW     ACT_2_ON           ; 42H  TURNS ON_OFF ACTUATOR 2
    DW     cACT_ENA           ; 43H  ARMS 28V ACTUATOR PWR SWITCH
    DW     cAUX_PWR_ENA       ; 44H  Turns ON_OFF 28V AUX OSCILLATOR for Motor, Actuator, I
    DW     ACT_RESET          ; 45H  Actuator reset sequence
    DW     HVNOMINAL          ; 46H *LFHVNMA Nom value for HV A
    DW     HVNOMINBL          ; 47H  LFHVNMB Nom value for HV B
    DW     NO_OPER            ; 48H  No Operation, undefined command
    DW     NO_OPER            ; 49H  No Operation, undefined command
    DW     MODE               ; 4AH *CRP_OVR CRP override
    DW     HV_A_CMD           ; 4BH *SET BITS FOR MCP-A HV BIAS
    DW     HV_B_CMD           ; 4CH  SET BITS FOR MCP-B HV BIAS
    DW     NO_OPER            ; 4DH  No Operation, undefined command
    DW     RAMPTIMEC          ; 4EH *HV_RAMP HV Ramp time constant
    DW     HVQ_ON             ; 4FH  switches MCP grid supply on
    DW     HV_I_MAX           ; 50H *Set HV current limit
    DW     HV_ARM             ; 51H *Enables HV_ON and HVQ_ON (in the DPU = software)
    DW     MODE               ; 52H  TURNS ON ION PUMP #1
    DW     MODE               ; 53H  TURNS ON ION PUMP #2
    DW     MODE               ; 54H  ENABLES ION PUMPS #1& #2
    DW     GDGBOFFY           ; 55H *TDC Begin Y interpolator offset Y_SHIFT_DAC
    DW     GDGEOFFY           ; 56H *TDC End Y interpolator  offset Y_SHIFT_DAE
    DW     GDGBSCLY           ; 57H *TDC Begin Y interpolator scale Y_SCALE_DAC
    DW     GDGESCLY           ; 58H *TDC End Y interpolator scale Y_SCALE_DAE
    DW     MOTOR_CNTRL        ; 59H  Sets motor bridge switches (direction) avoid M1=M2=1
    DW     MOTOR_ENA          ; 5AH  ENABLES MOTOR BRIDGE SWITCHES (may start the motor)
    DW     MOTOR_ARM          ; 5BH  Allows MOTOR_ENA (in the DPU)
    DW     DOOR_I_MAX         ; 5CH  Set door current limit
    DW     NO_OPER            ; 5DH  No Operation, undefined command
    DW     NO_OPER            ; 5EH  No Operation, undefined command
    DW     NO_OPER            ; 5FH  No Operation, undefined command
    DW     MODE               ; 60H  Mask Box image operation
    DW     GDGBOFFX           ; 61H *TDC Begin X interpolator  offset [#DGBOFFX]
    DW     GDGEOFFX           ; 62H *TDC End X interpolator  offset [#DGEOFFX]
    DW     GDGBSCLX           ; 63H *TDC Begin X interpolator scale [#DGBSCLX]
    DW     GDGESCLX           ; 64H *TDC End X interpolator scale [#DGESCLX]
    DW     MODE               ; 65H  Mask Box image operation
    DW     HSTATE             ; 66H *Trigger shutdown, no auto restore
    DW     NOMSTATE           ; 67H *Restore HV saved during shutdown
    DW     MODE               ; 68H  Set SAA shutdown values to 80% of HV
    DW     END_COMMAND        ; 69H  Don't expect anymore commands, no reset
    DW     GOTO               ; 6AH *Jump to specified address and execute code there
    DW     CRPMODE            ; 6BH *CRP enable/disable
    DW     NO_OPER            ; 6CH  No Operation, undefined command
    DW     NO_OPER            ; 6DH  No Operation, undefined command
    DW     SAA_OVER           ; 6EH *day - Restore HV saved during shutdown
    DW     SAA_HERE           ; 6FH *night - Trigger shutdown, no auto restore
    DW     HARD_RESET         ; 70H *power-on reset
    DW     WD_STATE           ; 71H *enable/disable watchdog
    DW     NO_OPER            ; 72H  No Operation, undefined command
    DW     JMPCODE            ; 73H *jump code
    DW     ZAP_TS30           ; 74H *clear diags
    DW     SOFT_RESET         ; 75H *watchdog reset
    DW     SAFE_CONTROLS      ; 76H *Enter Idle state, HV off [#DSBITS5]
    DW     DEBUGG             ; 77H  No Operation, undefined command
    DW     NO_OPER            ; 78H  No Operation, undefined command
    DW     HKPTEST            ; 79H  Insert Timestamps in HKP Packet
    DW     ACTEL_RESET        ; 7AH  Toggle reset line to counter actel
    DW     TDC                ; 7BH  TDC debug dbb 4-14-00
    DW     JNTTEST            ; 7CH  Reserved for J&T
    DW     PROM_STATE         ; 7DH *Set PROM bit according to parameter
    DW     TDC_A_TEST         ; 7EH  dbb 4-1-00
    DW     HKPFULL            ; 7FH *Send full copy of Housekeeping Memory

    SPARE_CODE   6            ; insert nops to align the next section
$EJECT
;*****************************************************************************
;
; Subroutine  TST_TBL
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
TST_TBL:
    AJMP   DO_TEST0
    AJMP   DO_TEST1
    AJMP   DO_TEST2
    AJMP   DO_TEST3
    AJMP   DO_TEST4
    AJMP   DO_TEST5
    AJMP   DO_TEST6
    AJMP   DO_TEST7
    AJMP   DO_TEST8
    AJMP   DO_TEST9
    AJMP   DO_TESTA
    AJMP   DO_TESTB
    AJMP   DO_TESTC
    AJMP   DO_TESTD
    AJMP   DO_TESTE
    AJMP   DO_TESTF
    AJMP   DO_TEST10          ; ignore all others
    AJMP   DO_TEST11          ; ignore all others
    AJMP   DO_TEST12          ; ignore all others
    AJMP   DO_TEST13          ; ignore all others
    AJMP   DO_TEST14          ; ignore all others
    AJMP   DO_TEST15          ; ignore all others
    AJMP   DO_TEST16          ; ignore all others
    AJMP   DO_TEST17          ; ignore all others
    AJMP   DO_TEST18          ; ignore all others
    AJMP   DO_TEST19          ; ignore all others
    AJMP   DO_TEST1A          ; ignore all others
    AJMP   DO_TEST1B          ; ignore all others
    AJMP   DO_TEST1C          ; ignore all others
    AJMP   DO_TEST1D          ; ignore all others
    AJMP   DO_TEST1E          ; ignore all others
    AJMP   DO_TEST1F          ; ignore all others
TESTCMD:                      ; enter test mode. (dump pseudo telemetry ...)
    MOV    A,R0               ; Get parameter
    ANL    A,#01Fh            ; mod 16, limited jump table size
    CLR    C                  ; make sure 0 is shifted in while multiplying
    RLC    A                  ; multiply by two for jump table
    MOV    DPTR,#TST_TBL      ; Use jump table below to select command
    JMP    @A+DPTR

    SPARE_CODE   7            ; insert nops to align the next section
$EJECT
DO_TEST0:
DO_TEST1:                     ; Reboot by triggering watchdog reset
DO_TEST2:                     ; Reboot by toggling
    JMP    SOFT_RESET
DO_TEST3:
    JMP    HARD_RESET
DO_TEST4:
    MOV    CCAPM4,#0h         ; Disable Watchdog
    RET
DO_TEST5:
    MOV    CCAPM4,#48h        ; CCAPM4 setup as WatchDog Timer
    RET
DO_TEST6:                     ; jump to upper/lower code
    JMP    JMPCODE
DO_TEST7:
    RET
DO_TEST8:
    JMP    INIT_T0            ; turns interrupts off, dbb 11-30-98
DO_TEST9:
    JMP    ZAP_TS30
DO_TESTA:
    CPL    EA                 ; dis/enable interrupts, dbb 11-30-98
DO_TESTB:                     ; Control the direction up/down
DO_TESTC:                     ; change TH1, up/down by 1
DO_TESTD:
DO_TESTE:
DO_TESTF:
    RET
DO_TEST10:
    CPL    ESS_FLAG           ; packet size control for GALEX.
    RET
DO_TEST11:
    CPL    SAA_FLAG
    RET
DO_TEST12:
    CPL    CTRS_FLAG          ; Ramp flag side A
    RET
DO_TEST13:
    CPL    PHS_FLAG           ; Ramp flag side B
    RET
DO_TEST14:
    CPL    MUX_FLAG           ; controls MUX access for testing
    RET
DO_TEST15:
    CPL    LATCH_FLAG         ; was JMP    REGISTERS
DO_TEST16:
    CALL   vSETMUX            ; set according to param1
    RET
DO_TEST17:
    JMP    SETPROJECT         ; detect presence of B detector
DO_TEST18:
    CPL    PROJECT
    RET
DO_TEST19:
    MOV    A,#0FFh
    MOV    R0,A
    AJMP   DO_TEST20
DO_TEST1A:
    CLR    A
    MOV    R0,A
    AJMP   DO_TEST20
DO_TEST1B:                    ; report any arbitrary diagnostic
    MOV    DPL,R1             ; from param1
    MOV    DPH,R3             ; from param1
    CALL   REPORT_DIAGNOSTIC  ; Record this dreadful event
    RET
DO_TEST1C:
    MOV    A,R1               ; R1 selects an item from this table
    CLR    C
    RLC    A                  ; multiply by 2
    MOV    DPTR,#SENSOR_INDEX ; look at table of MUX specs
    MOVC   A,@A+DPTR          ; get high byte of pointer
    PUSH   ACC                ; save DH
;    FETCHX RS17_PAR1
    MOV    A,R1               ; get index again
    CLR    C
    RLC    A                  ; multiply by 2
    INC    ACC                ; point to low byte of pointer
    MOV    DPTR,#SENSOR_INDEX ; look at table of MUX specs
    MOVC   A,@A+DPTR          ; get low byte of pointer
    MOV    DPL,A              ; load low pointer
    POP    DPH                ; load high pointer
    PUSH   DPH		      ;  and save on stack so we can pop them later
    PUSH   DPL		      ;  and save on stack so we can pop them later
;
    CALL   GETMUXINFO         ; do mux activity on item pointed-to
    SETB   MUX_FLAG           ; controls MUX access for testing
    SPARE_CODE 3
;
    CLR    A
    STOREX RS17_PAR2          ; external data
    INC    A
    INC    A
    STOREX RS17_PAR0          ; external data
    POP    ACC                ; DPH
    STOREX RS17_PAR1          ; external data
    POP    ACC                ; DPL
    STORNEXT
    LJMP   PROGRAMMONITOR     ; 01H  index,addr,bank
    RET
DO_TEST1D:
    RET
DO_TEST1E:
    RET
DO_TEST1F:
    RET
DO_TEST20:                    ;       not accessable... code storage only
    MOV    DPTR,#CONTROLS1+0FFh ; addresses of bits encoded in DPTR
; The LOCK bit was implemented incorrectly in hardware, so all accesses to the
; Controls registers will be bracketed by asserting and deasserting the LOCK
; bit.  The LOCK is set to its locked state after the either control register
; is updated.
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    JMP    CPROMS             ; and set it according to parameter R0
    RET                       ; do nothing
$EJECT
;*****************************************************************************
;
; Subroutine  JNTCODE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
JNTCODE:
    MOV    DPH,#0C0h          ; high-byte of jump address into DPTR
    MOV    DPL,#0             ; and the rest of the destination pointer
    JMP    GOTO               ;
$EJECT
;*****************************************************************************
;
; Subroutine  DEBUGG
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
DEBUGG:
   CPL     fFORMAT            ; fFORMAT used in general debugging...
   RET
$EJECT
;*****************************************************************************
;
; Subroutine  END_COMMAND
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
END_COMMAND:
    JMP    REENTRY            ; restart without reset
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  GOTO
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v17057 dbb 07-18-00 Removed CLR CMD_ERR, changed meaning of flag
;*****************************************************************************
; Tested! dbb 5-17-96
GOTO:                         ; Jump to specified address
                              ;   and execute code there
;    CLR    CMD_ERR            ; avoid Detect command fail
    MOV    DPH,R2             ; high-byte of jump address into DPTR
    MOV    DPL,R0             ; and the rest of the destination pointer
    CLR    A                  ; offset by zero
    JMP    @A+DPTR            ; Goodbye!
$EJECT
;*****************************************************************************
;
; Subroutine  TEST
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
TEST:                         ; enter test mode. (dump pseudo telemetry ...)
    JMP   TESTCMD

vSETMUX:                      ; set according to param1
    JMP   SETMUX
$EJECT
;*****************************************************************************
;
; Subroutine  NO_OPER
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
NO_OPER:                      ; No Operation, can be used as
                              ;   Ping to check aliveness
    CLR    A                  ; Acknowledge NOP command, or undefined command
    SETB   NOP_FLAG           ; it happened...
    RET                       ; NO_OPER
$EJECT
;*****************************************************************************
;
; Subroutine  CMD_ILLEGAL
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
CMD_ILLEGAL:                  ; Report this to the proper authorities!
    REPORT_DIAG DIAG0011      ; CMD_ILLEGAL!
    RET                       ; CMD_ILLEGAL
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  MODE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
MODE:                         ; set observation mode
    REPORT_DIAG DIAG0014      ; Function not implemented!
    RET                       ; Go back to caller to COMMAND_VECTOR
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  CHECK
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; I'm not sure what was intended here, but this command simply copies the
; contents of internal memory into some transmit subaddress space
CHECK:                        ; Check a value (returned as a monitor)
    FETCHX RS17_PAR0          ; internal memory pointer in param 0
    MOV    R0,A               ; this is a byte activity only
    MOV    A,@R0              ; whatever R0 is pointing at...
    STOREX tPOINTMON          ; LFDPEEK = POINTMON
    RET                       ; Go back to caller to COMMAND_VECTOR
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  WD_STATE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
WD_STATE:
    MOV    A,R0               ; parameter to accum for zero test
    JNZ    WD_ENABLE          ; and then do the right thing.
WD_DISABLE:
    MOV    CCAPM4,#0h         ; Disable Watchdog
    CLR    WATCHDOG_STATE
    RET
WD_ENABLE:
    JMP    INIT_WD
    MOV    CCAPM4,#48h        ; CCAPM4 setup as WatchDog Timer
    RET                       ; WD_STATE
    NOP                       ; word alignment

;*****************************************************************************
UPDATE_BIT MACRO bitflag
    LOCAL FLAGUPDATED
    CLR    bitflag            ; reset state flag
    MOV    A,R0               ; parameter in R0
    JZ     FLAGUPDATED
    SETB   bitflag            ; set state flag
FLAGUPDATED:
    ENDM                      ; UPDATE_BIT

;*****************************************************************************
CLOCK_CDC    MACRO  CDC_NXT
    SETB   C		      ;  clock up
    MOV    CDC_NXT,C          ; and clock it in -- up
    CLR    C		      ;  clock down
    MOV    CDC_NXT,C          ; and clock it in -- down
    ENDM                      ; CLOCK_CDC

;*****************************************************************************
SEND_N_CLOCK MACRO cdc_data, cdc_next
    LOCAL SNC_OUTLOOP
                              ; A holds bytes to send out
    MOV    R1,#8              ; 8-bits per byte
SNC_OUTLOOP:
    RLC    A                  ; put the bit to transmit in C
    MOV    cdc_data,C         ; and send it
    CLOCK_CDC %cdc_next       ; and clock it in
    DJNZ   R1,SNC_OUTLOOP     ; do all 8 bits in command part
    ENDM                      ; SEND_N_CLOCK
$EJECT
;*****************************************************************************
;
; Subroutine                                ; Parameters
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
;  This subroutine writes to the CDC
CDC_OUT MACRO cdc_dav, cdc_data, cdc_next
    LOCAL CDC_C_OUTLOOP, CDC_D_OUTLOOP, SKIPLOOKUP
                              ; Parameters:
                              ;   A  holds CDC command
                              ;   R0 holds data parameter

$IF XSPEC
    JB     MUX_FLAG,SKIPLOOKUP ; this use of the mux flag evolved out of
                              ; a laboratory necessity to test the TDC
                              ; controller by specifying the selector and
                              ; data.  The MUX Flag also disables readback
                              ; from hardware.
    CALL   PROJECTCDC         ; select the correct cdc command, returned in A
SKIPLOOKUP:
$ELSE
    SPARE_CODE 3              ; account for xspec code left out of flight
    CALL   PROJECTCDC         ; select the correct cdc command, returned in A
$ENDIF

;
; dbb blew it here!  He thought the DAV was a gate when specifying the
;  behavior of the DAV line.  In FUSE the DAV was actually a strobe. Oops.
;  The logic here is supposed to support FUSE-like behavior, ie, if you
;  want DAV to be a strobe, you zap the next word.  By default its a gate.
    SETB   cdc_dav            ; set the data valid bit -- Open GATE.

    SEND_N_CLOCK %cdc_data, %cdc_next

    MOV    A,R0               ; get the data part
    SEND_N_CLOCK %cdc_data, %cdc_next

    SETB   cdc_dav            ; set the data valid bit
    NOP                       ; 0.75us
    CLR    cdc_dav            ; reset the data valid bit
    CLR    cdc_data           ; reset the data valid bit
;
    ENDM                      ; CDC_OUT
$EJECT
;*****************************************************************************
ARM_CHECK MACRO armbit, diagnostic
    LOCAL ARMED_OK
    CALL   CHECKBOOT          ; check for boot mode violation
    MOV    A,R0               ; let's look at parameter,
    JZ     ARMED_OK             ; if zero, then armbit not required
    JB     armbit,ARMED_OK    ; if armbit is set then we are properly armed
    REPORT_DIAG diagnostic ; Report
    RET                       ; then return to caller COMMAND_VECTOR
ARMED_OK:                     ; continue as properly armed
    ENDM                      ; ARM_CHECK
$EJECT
;*****************************************************************************
;
; Subroutine  aARMED
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; The idea here is we not only set the armbit to the appropriate state, but
;  also report a diagnostic if the bit is already in that state.  This may
;  help us catch a bug or two.
SET_ARM_BIT MACRO armbit,alreadyArmed,alreadyDisarmed
    LOCAL BIT_DISARM,BIT_DONE,aARMED
    MOV    A,R0               ; fetch parameter
    JZ     BIT_DISARM         ; means disarm, else...
    JB     armbit,aARMED      ; test for repeat command
    SETB   armbit             ; turn on arm bit to enable some activity
    SJMP   BIT_DONE
aARMED:
    REPORT_DIAG alreadyArmed  ; repeat command!
    SJMP   BIT_DONE
BIT_DISARM:
    ARM_CHECK armbit,alreadyDisarmed
    CLR    armbit             ; turn off arm bits to disallow some activity
BIT_DONE:
    ENDM                      ; SET_ARM_BIT
$EJECT
;*****************************************************************************
;
; Subroutine  TDC_A_TEST
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
$IF XSPEC
; Added for debug testing dbb 4-1-00
TDC_A_TEST:
    MOV    A,#TDC_STIM_ACTIVE ; point to a command and drop thru to send it
                              ; out the A-side TDC.
$ELSE
    SPARE_CODE 2              ; account for xspec code left out of flight
$ENDIF
$EJECT
;*****************************************************************************
;
; Subroutine  CDC_A_OUT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
;  This subroutine writes to the CDC-A
CDC_A_OUT:                    ; Parameters:
                              ;   A  holds CDC command
                              ;   R0 holds data parameter
    CDC_OUT CDC_A_DAV, CDC_A_DATA, CDC_A_NXT
    RET                       ; CDC_A_OUT
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  CDC_B_OUT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
;  This subroutine writes to the CDC-B
CDC_B_OUT:                    ; Parameters:
                              ;   A  holds CDC command
                              ;   R0 holds data parameter
    CDC_OUT CDC_B_DAV, CDC_B_DATA, CDC_B_NXT
    RET                       ; CDC_B_OUT
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  SETBIT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; The LOCK bit was implemented incorrectly in hardware, so all accesses to the
; Controls registers will be bracketed by asserting and deasserting the LOCK
; bit.  Note that the LOCK is not reset in this routine, only set to its
; locked state after either control register is updated.
;*****************************************************************************
; This routine does whatever is required to set the output bit to 0 or 1
;  depending on the parameter R0.  DPTR points to the address of the bit.
;  The bit is output of Board C, Controls 1 or 2.
SETBIT:                       ; DPTR must be setup prior to entry here
    PUSH   6
    PUSH   7
    MOV    A,DPL              ; these bits are the ones to change
    MOV    R7,A               ; stash a copy of the mask in a register
    CPL    A                  ; flip bits in case we are resetting
    MOV    R6,A               ; stash a copy of the mask in a register
    CLR    A                  ; A <-- 0
    MOV    DPL,A              ; normalize pointer to base of register addr
    MOV    A,R0               ; low-byte of parameter, 0=clr, else set, bits
    JZ     RESETBIT
; So we are going to set the designated bit(s)
    MOVX   A,@DPTR            ; fetch the current contents of the control reg
    ORL    A,R7               ; set the bit(s)
    JMP    CLEANUP
RESETBIT:
    MOVX   A,@DPTR            ; fetch the current contents of the control reg
    ANL    A,R6               ; reset the bit(s)
CLEANUP:
    MOVX   @DPTR,A            ; update the control register
; The LOCK bit was implemented incorrectly in hardware, so all accesses to the
; Controls registers will be bracketed by asserting and deasserting the LOCK
; bit.  The LOCK is set to its locked state after the either control register
; is updated.
    SETB   C1_LOCK            ; make sure Controls 1 are locked
    POP    7
    POP    6
    RET                       ; SETBIT, Go back to caller of COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  PROM_STATE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; NOTE! No telemetry item for this in FUSE! dbb 2-5-99
; Talkbacks in hkp for both projects.
PROM_STATE:
    CALL   CHECKBOOT          ; check for boot mode violation
    JB     PROJECT,GPROMS     ; PROM bits are laid out differently by project
    MOV    DPTR,#CONTROLS1+0C0h ; addresses of bits encoded in DPTR
    AJMP   CPROMS
GPROMS:
    MOV    DPTR,#CONTROLS1+084h ; addresses of bits encoded in DPTR
CPROMS:
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    JMP    SETBIT             ; and set it according to parameter R0
    NOP                       ; word alignment

;*****************************************************************************
PROM_ON:
    MOV    R0,#0              ; Power On
    SPARE_CODE 2
    AJMP   PROM_STATE         ; PROM_ON

;*****************************************************************************
PROM_OFF:
    MOV    R0,#1              ; Power Off
    SPARE_CODE 2
    AJMP   PROM_STATE         ; PROM_OFF
$EJECT
;*****************************************************************************
;
; Subroutine  skip_v_save
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
RMOVX  MACRO  dest, rsrc
    MOV    A,rsrc             ; load the thing to be saved there
    STOREX dest               ; copy A to external dest
    ENDM                      ; RMOVX

;*****************************************************************************
HV_x_DAC MACRO s_saved, vset, rflag, hvtarget, hvrampcnt, hvvsetting
LOCAL skip_v_save, NORAMP, NOSET
    CALL   CHECKBOOT          ; check for boot mode violation
    JBC    F0,skip_v_save     ; shutdown avoids updating VSET
    RMOVX  s_saved,R0         ; put parameter hv setting in sx_saved variable
skip_v_save:
; next, check if we are doing HV ramping
    JB     rflag,NORAMP       ; if we are already ramping, just do it.
    MOV    A,HVRAMP           ; an internal HVRAMP variable, 0 == disabled
    JZ     NORAMP             ; old way, hv set directly when rflag reset
    RMOVX  hvrampcnt,HVRAMP   ; put ramp slope in hvrampcnt
    RMOVX  hvtarget,R0        ; put parameter value in hvtarget for ramp
    JZ     NORAMP             ; if target HV is 0, we just do it anyway
    SETB   rflag              ; let mainroutine update hv ramp
    AJMP   NOSET              ;  -- don't do it here!

NORAMP:                       ; set hv directly
    RMOVX  vset,R0            ; put parameter value in DAC
    STOREX hvvsetting         ; and save it somewhere in external memory
NOSET:
    ENDM                      ; HV_DAC
$EJECT
;*****************************************************************************
;
; Subroutine  HV_A_UPDATE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
; v17047 dbb 07-01-00 Added segment select to support side B
;                                                                            *
;*****************************************************************************
HV_A_UPDATE:
    HV_x_DAC SA_SAVED,VSETA,CTRS_FLAG,tLFHVTGTA,HVArampcnt,HVAsetting
    RET

HV_A_CMD:                     ; SET BITS FOR MCP-A HV BIAS
    FETCHX  RS17_PAR1         ; look at segment select
    JNZ     HV_B_CMD
    MOV    A,#6               ; Event H
    STOREX LFHSTATE           ; HV State always init to 0 on any reset
    SETB   RAMPACMD           ; speed up ramp algorithm
    CLR    CTRS_FLAG          ; make sure ramp happens only on command
    JNB    SA_STATE,HV_A_DAC
    PUSH   0
    CALL   SAA_A_INIT         ; reset the SA flag
    POP    0

HV_A_DAC:                     ; SET BITS FOR MCP-A HV BIAS
    JB     PROJECT,SDMCOMPARE ; if GALEX, there is no hardware here,
    CALL   HV_A_UPDATE
    STOREX tHV_A_SET          ; dbb 7-21-98 ; dbb 12-24-98
    RET                       ; HV_A_DAC

SDMCOMPARE:
    PUSH   0                  ; save the command parameter value
    FETCHX tHV_A_MAX          ; previous commanded HV set
    CJNE   A,0,SDMUPPER       ; compare parameter setting to hv_max
    AJMP   SDMLOWER           ; if hv max equals setting, do nothing
SDMUPPER:
    JNC    SDMLOWER           ; if max setting is less than hv, just record hv
    MOV    R0,A               ; use MAX value instead of commanded setting
SDMLOWER:
    CALL   HV_A_UPDATE
    POP    ACC                ; get the command parameter value
    STOREX tHV_A_SET
    RET                       ; Go back to caller to COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  HV_B_UPDATE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HV_B_UPDATE:
    HV_x_DAC SB_SAVED,VSETB,PHS_FLAG,tLFHVTGTB,HVBrampcnt,HVBsetting
    RET

HV_B_CMD:                     ; SET BITS FOR MCP-B HV BIAS
    MOV    A,#6               ; Event H
    STOREX LFHSTATE           ; HV State always init to 0 on any reset
    SETB   RAMPBCMD           ; speed up ramp algorithm
    CLR    PHS_FLAG           ; make sure ramp happens only on command
    JNB SB_STATE,HV_B_DAC
    PUSH   0
    CALL   SAA_B_INIT         ; reset the SB flag
    POP    0

HV_B_DAC:                     ; SET BITS FOR MCP-B HV BIAS
    JB     PROJECT,SKMCOMPARE ; if GALEX, there is no hardware here,
    CALL   HV_B_UPDATE
    STOREX tHV_B_SET          ; dbb 7-21-98 ; dbb 12-24-98
    RET

SKMCOMPARE:
    FETCHX tHV_B_MAX          ; previous commanded HV set
    CJNE   A,0,SKMUPPER       ; compare parameter setting to hv_max
    MOV    A,R0               ; prepare to save parameter in case we jump
    AJMP   SKMLOWER           ; if hv max equals setting, do nothing
SKMUPPER:
    MOV    A,R0               ; prepare to save parameter in case we jump
    JC     SKMLOWER           ; if max setting is less than hv, just record hv
    CALL   HV_B_UPDATE
SKMLOWER:
    STOREX tHV_B_SET
    RET                       ; Go back to caller to COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  HV_A_MAX
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HV_A_MAX:                     ; set maximum setting for HV A
    FETCHX  RS17_PAR1         ; look at segment select
    JNZ     HV_B_MAX
    CALL   CHECKBOOT          ; check for boot mode violation
    JB     PROJECT,SIMCOMPARE ; if GALEX, there is no hardware here,
                              ;   (via DAC and clamping diode)
    RMOVX  VMAXA,R0           ; put parameter value in DAC
    STOREX tHV_A_MAX
    RET                       ; Go back to caller to COMMAND_VECTOR

; In GALEX there is no hardware to support the MAX clamp, so we
; simulate it here in software.  If the HV_MAX setting is above a previous
; HV_DAC setting, then we set the HV to the HV_DAC setting; if the HV_MAX
; setting is below the previous HV_DAC setting, then we set HV target to
; HV_MAX.  In any case we record the HV_MAX setting for use when commanding
; a new HV_DAC setting.
SIMCOMPARE:
    PUSH   0                  ; save the command parameter value
    FETCHX tHV_A_SET          ; previous commanded HV set
    CJNE   A,0,SIMUPPER
    AJMP   SIMLOWER           ; if hv max is already set, do nothing
SIMUPPER:
    JNC    SIMLOWER           ; if hv setting is less than max, just record max
    MOV    R0,A               ; get HV_A_SET
SIMLOWER:
    CALL   HV_A_UPDATE
    POP    ACC                ; get the command parameter value
    STOREX tHV_A_MAX          ; save command parameter in talkback
    RET                       ; Go back to caller to COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  HV_B_MAX
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HV_B_MAX:                     ; set maximum setting for HV B
    CALL   CHECKBOOT          ; check for boot mode violation
    JB     PROJECT,SJMCOMPARE ; if GALEX, there is no hardware here,
                              ;   (via DAC and clamping diode)
    RMOVX  VMAXB,R0           ; put parameter value in DAC
    STOREX tHV_B_MAX
    RET                       ; Go back to caller to COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  SJMCOMPARE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; In GALEX there is no hardware to support the MAX clamp, so we
; simulate it here in software.  If the HV_MAX setting is above a previous
; HV_DAC setting, then we set the HV to the HV_DAC setting; if the HV_MAX
; setting is below the previous HV_DAC setting, then we set HV target to
; HV_MAX.  In any case we record the HV_MAX setting for use when commanding
; a new HV_DAC setting.
SJMCOMPARE:
    FETCHX tHV_B_SET          ; previous commanded HV set
    CJNE   A,0,SJMUPPER
    MOV    A,R0               ; prepare to save parameter in case we jump
    AJMP   SJMLOWER           ; if hv max is already set, do nothing
SJMUPPER:
    MOV    A,R0               ; prepare to save parameter in case we jump
    JC     SJMLOWER           ; if hv setting is less than max, just record max
    CALL   HV_B_UPDATE
SJMLOWER:
    STOREX tHV_B_MAX
    RET                       ; Go back to caller to COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  HVQ_ON
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HVQ_ON:                       ; switches MCP grid supply on
    ARM_CHECK HV_ARM_BIT,DIAG0008 ; Arming bit required
    UPDATE_BIT HVQ_ON_BIT     ; record state
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    MOV    DPTR,#HV_ENA_2     ; address of bit  dbb 10-30-96
    JMP    SETBIT             ; and set it according to parameter R0
$EJECT
;*****************************************************************************
;
; Subroutine  HV_ON
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HV_ON:                        ; switches HV BIAS supplies on after
                              ;   writing 0 to the DAC.
    ARM_CHECK HV_ARM_BIT,DIAG0008 ; Arming bit required
    UPDATE_BIT HV_ON_BIT      ; record state
; We turn off the DACs, strangely enough, when we enable them here
    PUSH   0                  ; save parameter
    MOV    R0,#0              ; Turn off DACs
    CALL   HV_A_DAC           ; SET BITS FOR MCP-B HV BIAS
    CALL   HV_B_DAC           ; SET BITS FOR MCP-B HV BIAS
    CALL   HV_A_MAX           ; SET BITS FOR MCP-B HV BIAS
    CALL   HV_B_MAX           ; SET BITS FOR MCP-B HV BIAS
    POP    0                  ; restore parameter
;
    MOV    A,R0               ; get parameter 1=on, 0=off
    JZ     HVST0              ; we turn on only
;    FETCHX LFHSTATE
;    JNZ    NSPECIAL
    MOV    A,#7               ; Special case
    STOREX LFHSTATE           ; HV State always init to 0 on any reset
    JMP    NSPECIAL
HVST0:
    CLR    A
    STOREX LFHSTATE           ; HV State always init to 0 on any reset
    JMP    NSPECIAL
NSPECIAL:
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    CLR    HVSDA              ; reset flags used for HV_I checking
    CLR    HVSDB              ; reset flags used for HV_I checking
    MOV    DPTR,#HV_ENA_1     ; address of bit
    JMP    SETBIT             ; and set it according to parameter R0
$EJECT
;*****************************************************************************
;
; Subroutine  HV_ARM
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HV_ARM:                       ; Enables HV_ON and HVQ_ON (in the software)
    SET_ARM_BIT HV_ARM_BIT,DIAG0009,DIAG000A
    RET                       ; Go back to caller to COMMAND_VECTOR

    FROGS_CODE 39
$EJECT
;*****************************************************************************
;
; Subroutine  cFORMAT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
cFORMAT:                      ; set data format on RS-422 (for calibration)
    UPDATE_BIT fFORMAT
    RET                       ; Go back to caller to COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  cSCRUB_ON
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; This command and its bit are no longer used.  Not sure what the h/w does...
cSCRUB_ON:                    ; TURNS ON_OFF SCRUB LAMP
    MOV    DPTR,#SCRUB_ON     ; address of bit
    JMP    SETBIT             ; set it, then return to caller

STIM_ON_OFF_A:                ; TURNS TDC-B STIMULUS GENERATOR ON_OFF
    MOV    R1,#0              ; select side A
    JMP    LFGSTIM
STIM_ON_OFF_B:                ; TURNS TDC-B STIMULUS GENERATOR ON_OFF
    MOV    R1,#1              ; select side B
    JMP    LFGSTIM
$EJECT
;*****************************************************************************
;
; Subroutine  PROJECTCDC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; Convert cdc index to command, accounting for the different layouts in G/C
PROJECTCDC:
    PUSHDPTR                  ; DPTR probably holds the dest addr, save it
    MOV    DPTR,#CDCTABLE     ; load both DPL and DPH
    MOV    DPL,A              ; index is lower byte of pointer to table
    JNB    PROJECT,COSCDC
    INC    DPTR               ; point to correct COS/GALEX item
COSCDC:
    CLR    A                  ; zero
    MOVC   A,@A+DPTR          ; this could come from PROM or code RAM
    POPDPTR                   ; restore after use above
    RET                       ; PROJECTCDC convert index to command
$EJECT
;*****************************************************************************
;
; Subroutine  HVNOMINAL
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v17051 dbb 07-10-00 Removed erroneous HSTATE update
;*****************************************************************************
HVNOMINAL:
    CALL   CHECKBOOT          ; check for boot mode violation
    MOV    A,R0               ; parameter byte for nominal setting
    CJNE   R1,#0,HVNOMINBL    ; R1 selects side A or B
    STOREX LFHVNOMA           ; stored in housekeeping
    RET                       ; HVNOMINBL
HVNOMINBL:
    STOREX LFHVNOMB           ; stored in housekeeping
    RET                       ; HVNOMINBL
$EJECT
;*****************************************************************************
;
; Subroutine  RAMPTIMEC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
RAMPTIMEC:
    CALL   CHECKBOOT          ; check for boot mode violation
    MOV    HVRAMP,R0          ; put command parameter value in HVRAMP
    RET                       ; RAMPTIMEC
$EJECT
;*****************************************************************************
;
; Subroutine  CRPMODE
;                         6BH *CRP enable/disable
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
CRPMODE:
; This routine does nothing?  dbb 7-10-00
    RET                       ; CRPMODE
$EJECT
;*****************************************************************************
;
; Subroutine  SELECT_CDC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; The CDC is an old name for the TDC or Digitizer controller, preserved here
; to refer to the controller itself.  cdc commands are shifted out across the
; 3-wire interface to the TDC/Digitizer.
;*****************************************************************************
SELECT_CDC:
    CJNE   R1,#0,B_SIDE       ; R1 selects side A or B
    AJMP   CDC_A_OUT          ; R0 holds parameter
B_SIDE:
    AJMP   CDC_B_OUT          ; R0 holds parameter

;*****************************************************************************
CDC_SET MACRO pattern
    MOV    A,#LOW(pattern)    ; command to CDC: A holds index to table of cdc
    AJMP   SELECT_CDC         ; select the channel to output command and data
    ENDM                      ; CDC_SET
$EJECT
;*****************************************************************************
;
; Subroutine  LFGSTIM
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; R0 contains the parameter to the STIM command, which is sent as-is to the
; Digitizer/TDC as the lower byte of the stim command.
LFGSTIM:
    MOV    A,R0               ; look at setting parameter
    CJNE   R1,#0,STIM_B_SIDE  ; R1 selects side A or B
    STOREX tDGSTIM            ; put the parameter value in talkback STIM_A
    AJMP   STIM_A_SIDE        ; then send stim command out correct channel
STIM_B_SIDE:
    STOREX tDGSTIM+1          ; put the parameter value in talkback STIM_B
STIM_A_SIDE:
    CDC_SET STIM_CONTRL

;*****************************************************************************
GDGEOFFX:
    CDC_SET X_SHIFT_DAE

;*****************************************************************************
GDGESCLX:
    CDC_SET X_SCALE_DAE

;*****************************************************************************
GDGEOFFY:                     ; Y_SHIFT_DAE
    CDC_SET Y_SHIFT_DAE

;*****************************************************************************
GDGESCLY:                     ; Y_SCALE_DAE:
    CDC_SET Y_SCALE_DAE
$EJECT
;*****************************************************************************
;
; Subroutine  LWR_T_THR_A
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
LWR_T_THR_A:                  ; SET BITS FOR TDC-A LOWER TIME (X) THRESHOLD
    FETCHX  RS17_PAR1         ; look at segment select
    JNZ     LWR_T_THR_B
    FETCHX  RS17_PAR2         ; look at axis select
    JNZ     WYE_T_THR_A
    CDC_SET LWR_T_THR

;*****************************************************************************
LWR_T_THR_B:                  ; SET BITS FOR TDC-B LOWER TIME (X) THRESHOLD
    FETCHX  RS17_PAR2         ; look at axis select
    JNZ     WYE_T_THR_B
    MOV    R1,#1              ; select side B
    CDC_SET LWR_T_THR

WYE_T_THR_A:                  ; SET BITS FOR TDC-A LOWER TIME (Y) THRESHOLD
GDGTTY:                       ; WYE_T_THR
    CDC_SET WYE_T_THR

;*****************************************************************************
WYE_T_THR_B:                  ; SET BITS FOR TDC-B LOWER TIME (Y) THRESHOLD
    MOV    R1,#1              ; select side B
    CDC_SET WYE_T_THR

;*****************************************************************************
LWR_Q_THR_A:                  ; SET BITS FOR TDC-A LOWER CHARGE (Y) THRESHOLD
    FETCHX  RS17_PAR1         ; look at segment select
    JNZ     LWR_Q_THR_B
    CDC_SET LWR_Q_THR

;*****************************************************************************
LWR_Q_THR_B:                  ; SET BITS FOR TDC-B LOWER CHARGE (Y) THRESHOLD
    MOV    R1,#1              ; select side B
    CDC_SET LWR_Q_THR

;*****************************************************************************
UPR_Q_THR_A:                  ; SET BITS FOR TDC-A UPPER CHARGE (Y) THRESHOLD
    FETCHX  RS17_PAR1         ; look at segment select
    JNZ     UPR_Q_THR_B
    CDC_SET UPR_Q_THR

;*****************************************************************************
UPR_Q_THR_B:                  ; SET BITS FOR TDC-B UPPER CHARGE (Y) THRESHOLD
    MOV    R1,#1              ; select side B
    CDC_SET UPR_Q_THR
$EJECT
;*****************************************************************************
;
; Subroutine  X_OFFSET_A
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
X_OFFSET_A:                   ; SET BITS FOR TDC-A TIME (X) IMAGE POSITION OFFSET
    FETCHX  RS17_PAR1         ; look at segment select
    JNZ     X_OFFSET_B
    FETCHX  RS17_PAR2         ; look at axis select
    JNZ     Y_OFFSET_A
GDGBOFFX:
    CDC_SET X_SHIFT_DAC

;*****************************************************************************
X_OFFSET_B:                   ; SET BITS FOR TDC-B TIME (X) IMAGE POSITION OFFSET
    FETCHX  RS17_PAR2
    JNZ     Y_OFFSET_B
    MOV    R1,#1              ; select side B
    CDC_SET X_SHIFT_DAC

;*****************************************************************************
GDGBOFFY:                     ; Y_SHIFT_DAC
Y_OFFSET_A:                   ; SET BITS FOR TDC-A TIME (Y) IMAGE POSITION OFFSET
    CDC_SET Y_SHIFT_DAC

;*****************************************************************************
Y_OFFSET_B:                   ; SET BITS FOR TDC-B TIME (Y) IMAGE POSITION OFFSET
    MOV    R1,#1              ; select side B
    CDC_SET Y_SHIFT_DAC

;*****************************************************************************
X_SCALE_A:                    ; SET BITS FOR TDC-A TIME (X) SCALE FACTOR
    FETCHX  RS17_PAR1         ; check segment select (already in R1)
    JNZ     X_SCALE_B
    FETCHX  RS17_PAR2             ; check axis select
    JNZ     Y_SCALE_A
GDGBSCLX:
    CDC_SET X_SCALE_DAC       ; use pointer to table entry for actual opcode

;*****************************************************************************
X_SCALE_B:                    ; SET BITS FOR TDC-B TIME (X) SCALE FACTOR
    FETCHX  RS17_PAR2         ; check axis select
    JNZ     Y_SCALE_B
    MOV     R1,#1             ; select side B
    CDC_SET X_SCALE_DAC             ; use pointer to table entry for actual opcode

;*****************************************************************************
GDGBSCLY:                     ; Y_SCALE_DAC:
Y_SCALE_A:                    ; SET BITS FOR TDC-A TIME (X) SCALE FACTOR
    CDC_SET Y_SCALE_DAC             ; use pointer to table entry for actual opcode

;*****************************************************************************
Y_SCALE_B:                    ; SET BITS FOR TDC-B TIME (X) SCALE FACTOR
    MOV    R1,#1              ; select side B
    CDC_SET Y_SCALE_DAC       ; use pointer to table entry for actual opcode

;*****************************************************************************
BEG_WLK_A:                    ; SET BITS FOR TDC-A START (BEGIN) CFD
                              ;   WALK ADJUSTMENT
    FETCHX  RS17_PAR1         ; check segment select (already in R2)
    JNZ     BEG_WLK_B
    CDC_SET BEG_WALK          ; use pointer to table entry for actual opcode

;*****************************************************************************
;BEG_WLK_BY:
BEG_WLK_B:                    ; SET BITS FOR TDC-B START (BEGIN) CFD
                              ;   WALK ADJUSTMENT
    MOV     R1,#1             ; select side B
GDGBWKY:                      ; YBEGWALK:
    CDC_SET YBEGWALK          ; use pointer to table entry for actual opcode

;*****************************************************************************
END_WLK_A:                    ; SET BITS FOR TDC-A STOP (CFD) WALK ADJUSTMENT
    FETCHX  RS17_PAR1         ; check segment select (already in R2)
    JNZ     END_WLK_B         ; SET BITS FOR TDC-B STOP (CFD) WALK ADJUSTMENT
    CDC_SET END_WALK          ; use pointer to table entry for actual opcode

;*****************************************************************************
;END_WLK_BY:
END_WLK_B:                    ; SET BITS FOR TDC-B STOP (CFD) WALK ADJUSTMENT
    MOV     R1,#1             ; select side B
GDGEWKY:                      ; YENDWALK:
    CDC_SET YENDWALK          ; use pointer to table entry for actual opcode
$EJECT
;*****************************************************************************
;
; Subroutine  SPARE_O1_CMD
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SPARE_O1_CMD:                 ; spare
    RET                       ; SPARE_O1_CMD
$EJECT

;*****************************************************************************
; Command reports parameter at CrC display on eGSE
SPARE_S0_CMD:                 ; spare
    MOV    A,R0
    RET                       ; SPARE_S0_CMD

;*****************************************************************************
; This is a substitute for REDUCE/RESUME commands left out of eGSE
SPARE_S1_CMD:                 ; spare
    MOV    A,R0               ; look at parameter
    JZ     MUSTBEOVER
    JMP    SAA_HERE
MUSTBEOVER:
    JMP    SAA_OVER
    RET                       ; SPARE_S1_CMD

;*****************************************************************************
SPARE_S2_CMD:                 ; spare
    RET                       ; SPARE_S2_CMD
$EJECT
;*****************************************************************************
;
; Subroutines  ACT_1_ON, ACT_2_ON, cACT_ENA, ACT_RESET
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v17051 dbb 07-10-00 Changed AJMP GOOD_BOY  to  JMP GOOD_BOY
; v17058 dbb 07-19-00 Changed order of code in cACT_ENA
;*****************************************************************************
ACT_1_ON:                     ; TURNS ON_OFF ACTUATOR 1
    ARM_CHECK ACT_EN_BIT,DIAG0010
    UPDATE_BIT ACT1_BIT       ; record state
    MOV    DPTR,#sLFRACT2     ; address of bit
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    JMP    SETBIT             ; and set it according to parameter R0

;*****************************************************************************
ACT_2_ON:                     ; TURNS ON_OFF ACTUATOR 2
    ARM_CHECK ACT_EN_BIT,DIAG0010
    UPDATE_BIT ACT2_BIT       ; record state
    MOV    DPTR,#sLFRACT2     ; address of bit
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    JMP    SETBIT             ; and set it according to parameter R0

;*****************************************************************************
cACT_ENA:                     ; ARMS 28V ACTUATOR PWR SWITCH
    SET_ARM_BIT ACT_EN_BIT,DIAG000E,DIAG000F
    CALL   SET_MTR_TOUT       ; Three minutes, 30 seconds -- activity timeout
    MOV    DPTR,#sLFRACTEN    ; address of bit
    CLR    C1_LOCK            ; make sure Controls 2 unlocked dbb 3-21-00
    JMP    SETBIT             ; and set it according to parameter R0

;*****************************************************************************
ACT_RESET:                    ; Actuator Reset Sequence
    CALL   CHECKBOOT          ; check for boot mode violation
    MOV    A,R0               ; Check parameter, 0 means abort!
    JZ     ABORT_RESET
    FETCHX MUX                ; fetch door status bits
    ANL    A,#1               ; mask out all bits except LATCHED
    JNZ    NOT_FIRED          ; if the latch is set, actuators are not fired!
    MOV    R0,#1              ; turn on endswitch override
    CALL   MOTOR_SAFETY_OVERRIDE ; Overrides Endswitches
    MOV    R0,#2              ; command door open, must be previously armed.
    CALL   MOTOR_CNTRL        ; Sets motor bridge switches (direction)
    MOV    R0,#1              ; turn on motor!
    CALL   MOTOR_ENA          ; ENABLES MOTOR BRIDGE SWITCHES
    JMP    GOOD_BOY
ABORT_RESET:
    JMP    DOOR_STOP          ; Abort command! stop door motion!
NOT_FIRED:
    REPORT_DIAG DIAG0015      ; Latch already set, command ignored
GOOD_BOY:
    RET                       ; ACT_RESET
$EJECT
;*****************************************************************************
;
; Subroutine  cAUX_PWR_ENA
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
cAUX_PWR_ENA:                 ; Turns ON_OFF 28V AUX POWER
                              ;   for Motor, Actuator, Ion Pump
    CALL   CHECKBOOT          ; check for boot mode violation
    UPDATE_BIT AUX_PWR_ENA_BIT ; record state
    MOV    DPTR,#AUX_PWR_ENA  ; address of bit
    CLR    C1_LOCK            ; make sure Controls 2 unlocked dbb 3-21-00
    JMP    SETBIT             ; and set it according to parameter R0
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  SAA_CLMT_A
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAA_CLMT_X MACRO s_cnt_lmt,tmon ; SAA COUNT LIMIT FOR SEGMENT AUTO HV REDUCE
    MOV    A,R0               ; save low-byte of parameter value
    STOREX s_cnt_lmt
    MOV    A,R2               ; save high-byte of parameter value
    STORNEXT
    MOV    A,R0
    STOREX tmon
    MOV    A,R2
    STORNEXT
    ENDM                      ; SAA_CLMT_X

;*****************************************************************************
SAA_CLMT_A:                   ; SAA COUNT LIMIT FOR SEGMENT A AUTO HV REDUCE
    CJNE   R1,#0,SAA_CLMT_B     ; check parameter that specifies B side
    SAA_CLMT_X SA_CNT_LMT,tSAA_MAX_A   ; 28 08
    RET                       ; SAA_CLMT_A

;*****************************************************************************
SAA_CLMT_B:                   ; SAA COUNT LIMIT FOR SEGMENT B AUTO HV REDUCE
    SAA_CLMT_X SB_CNT_LMT,tSAA_MAX_B   ; 28 16
    RET                       ; SAA_CLMT_B
$EJECT
;*****************************************************************************
;
; Subroutine  SAA_HV_A
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAA_HV_A:                     ; SAA HV REDUCED VALUE SEGMENT A
    CJNE   R1,#0,SAA_HV_B     ; check parameter that specifies B side
    RMOVX SA_REDUCED,R0       ; Value of HV at Reduced level, Seg A
    STOREX tSAA_HV_A          ; dbb 5-29-97
    RET                       ; SAA_HV_A

;*****************************************************************************
SAA_HV_B:                     ; SAA HV REDUCED VALUE SEGMENT B
    RMOVX SB_REDUCED,R0       ; Value of HV at Reduced level, Seg B
    STOREX tSAA_HV_B          ; dbb 5-29-97
    RET                       ; SAA_HV_B
$EJECT
;*****************************************************************************
;
; Subroutine  SAA_HV_OVRD_A
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAA_HV_OVRD_A:                ; SAA HV AUTO SHUTDOWN OVER-RIDE SEGMENT A
    CJNE   R1,#0,SAA_HV_OVRD_B ; check parameter that specifies B side
    UPDATE_BIT SA_OVERRIDE    ; When set, AUTO reduce is disabled, Seg A
    CALL   SAA_A_INIT         ; resets SA_STATE
    RET                       ; SAA_HV_OVRD_A

;*****************************************************************************
SAA_HV_OVRD_B:                ; SAA HV AUTO SHUTDOWN OVER-RIDE SEGMENT B
    UPDATE_BIT SB_OVERRIDE    ; When set, AUTO reduce is disabled, Seg B
    CALL   SAA_B_INIT             ; resets SB_STATE
    RET                       ; SAA_HV_OVRD_B
$EJECT
;*****************************************************************************
;
; Subroutine  SAA_INT_A
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAA_INT_A:                    ; SAA INTEGRATION TIME FOR SEGMENT A
                              ;   AUTO HV REDUCE
    CJNE   R1,#0,SAA_INT_B    ; check parameter that specifies B side
    MOV    A,R0               ;  get lower byte of first parameter
    STOREX tLFPINTA           ; save in housekeeping
    STOREX SA_INT_TIME        ; SAA Integration Time for Count Limit, Seg A
    CALL   SAA_A_INIT         ; preset Trigger Table
    FETCHX RS17_PAR2             ; check max event counts per second
    JNZ    SAA_CLMT_A         ; SAA COUNT LIMIT FOR SEGMENT A AUTO HV REDUCE
    RET                       ; SAA_INT_A

;*****************************************************************************
SAA_INT_B:                    ; SAA INTEGRATION TIME FOR SEGMENT B
                              ;   AUTO HV REDUCE
    MOV    A,R0               ;  get lower byte of first parameter
    STOREX tLFPINTB           ; save in housekeeping
    STOREX SB_INT_TIME        ; SAA Integration Time for Count Limit, Seg B
    CALL   SAA_B_INIT         ; preset Trigger Table
    FETCHX RS17_PAR2             ; check max event counts per second
    JNZ    SAA_CLMT_B         ; SAA COUNT LIMIT FOR SEGMENT B AUTO HV REDUCE
    RET                       ; SAA_INT_B
$EJECT
;*****************************************************************************
;
; Subroutine  SAA_TOFF
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAA_TOFF:                     ; SAA SEGMENT AUTO HV REDUCE OFF PERIOD
    MOV    A,R0               ; low-byte of parameter
    STOREX SAA_TIMEOFF        ; Number of seconds(?) to hold HV Auto Reduce
    MOV    A,R2               ; high-byte of parameter
    STORNEXT                     ; point to high byte & save it
; Copy to telemetry, per abiad 5-29-97
    MOV    A,R0               ; low-byte of parameter
    STOREX tHV_AUTO_R           ; Number of seconds(?) to hold HV Auto Reduce
    MOV    A,R2               ; high-byte of parameter
    STORNEXT                  ; point to high byte & save it
    RET                       ; SAA_TOFF
$EJECT
;*****************************************************************************
;
; Subroutine  SAA_HERE, SAA_OVER
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAA_HERE:
    SETB   SAA_ASSERT         ; per abiad 5-29-97
    SETB   S_TRIGR            ; let CHECK_SAA trigger
    SETB   ASSERTED           ; suppress diag 22/23
    RET                       ; SAA_HERE

SAA_OVER:
    CLR    SAA_ASSERT         ; per abiad 5-29-97
    SETB   S_OVER             ; let CHECK_SAA timeout
    SETB   ASSERTED           ; suppress diag 22/23
    RET                       ; SAA_OVER
$EJECT
;*****************************************************************************
;
; Subroutine  MOTOR_CNTRL
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
MOTOR_CNTRL:                 ; Sets motor bridge switches (direction)
                              ;   avoid M1=M2=1
    ARM_CHECK MOTOR_ARM_BIT,DIAG000B
    CLR    C                  ; be sure subsequent left-shift inserts a zero
    MOV    A,R0               ; parameter should be 0,1,or2
    ANL    A,#3               ; modulo 4
    RLC    A                  ; multiply by two for jump table
    MOV    DPTR,#BRG_TBL      ; Use jump table below to select command
    JMP    @A+DPTR
    SPARE_CODE 29
    SPARE_CODE 14
BRG_TBL:                      ; Note Open/Close swapped per Abiad 1-15-97
    AJMP   BRIDGE_OFF         ; 00H  Set M1=0 M0=0
    AJMP   BRIDGE_CLOSE       ; 01H  Set M1=1 M0=0
    AJMP   BRIDGE_OPEN        ; 02H  Set M1=0 M0=1
    AJMP   BRIDGE_SHORT       ; 03H  Report this to the proper authorities!
BRIDGE_SHORT:
    JMP    CMD_ILLEGAL        ; 03H  Report this to the proper authorities!
BRIDGE_OPEN:
    ACALL  BRIDGE_OFF         ; make sure bridge is in a known state
    SETB   MOTOR_CNTRL_A_BIT  ; set state flag
    MOV    R0,#1              ; tell SETBIT to turn on a bit
    MOV    DPTR,#MA_CNTRL     ; turn on this bit
    JMP    SETBIT             ; set it, then return to caller
BRIDGE_CLOSE:
    ACALL  BRIDGE_OFF         ; make sure bridge is in a known state
    SETB   MOTOR_CNTRL_B_BIT  ; set state flag
    MOV    R0,#1              ; tell SETBIT to turn on a bit
    MOV    DPTR,#MB_CNTRL     ; turn on this bit
    JMP    SETBIT             ; set it, then return to caller
BRIDGE_OFF:
    CLR    MOTOR_CNTRL_A_BIT  ; reset state flag
    MOV    R0,#0              ; make sure parameter is really 0
    MOV    DPTR,#MA_CNTRL     ; turn off this bit
    CALL   SETBIT             ; and set it according to parameter R0
    CLR    MOTOR_CNTRL_B_BIT  ; reset state flag
    MOV    R0,#0              ; make sure parameter is really 0
    MOV    DPTR,#MB_CNTRL     ; turn off this bit
    JMP    SETBIT             ; set it, then return to caller
$EJECT
;*****************************************************************************
;
; Subroutine  MOTOR_ENA
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
MOTOR_ENA:                    ; ENABLES MOTOR BRIDGE SWITCHES
                              ;   (may start the motor)
    ARM_CHECK MOTOR_ARM_BIT,DIAG000B
    UPDATE_BIT MOTOR_ENA_BIT  ; record state
    CALL   SET_MTR_TOUT       ; Three minutes, 30 seconds -- activity timeout
    MOV    DPTR,#MOTOR_ON     ; address of bit
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    JMP    SETBIT             ; and set it according to parameter R0
$EJECT
;*****************************************************************************
;
; Subroutine  MOTOR_SAFETY_OVERRIDE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; NOTE! No telemetry item for this in FUSE! dbb 2-5-99
MOTOR_SAFETY_OVERRIDE:        ; Overrides Endswitches
                              ;   (may start the motor)
    CALL   CHECKBOOT          ; check for boot mode violation
    CALL   SET_MTR_TOUT             ; Three minutes, 30 seconds -- activity timeout
    MOV    DPTR,#OVERRIDE     ; address of bit
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    JMP    SETBIT             ; and set it according to parameter R0
$EJECT
;*****************************************************************************
;
; Subroutine  MOTOR_ARM
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
MOTOR_ARM:
    CALL   CHECKBOOT          ; check for boot mode violation
    SET_ARM_BIT MOTOR_ARM_BIT,DIAG000C,DIAG000D
;    CALL   SET_MTR_TOUT      ; Three minutes, 30 seconds -- activity timeout
; drop thru
    NOP                       ; word alignment
$EJECT
;*****************************************************************************
;
; Subroutine  SET_MTR_TOUT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
MTR_TOUT EQU 220              ; Three minutes, 30 seconds -- activity timeout
SET_MTR_TOUT:
    MOV    A,R0               ; get parameter
    JZ     SKIP_MTR_TOUT      ; set tout only if we are turning something on
    MOV    MOTORTMR,#MTR_TOUT ; Three minutes, 30 seconds -- activity timeout
SKIP_MTR_TOUT:
    RET                       ; Go back to caller to COMMAND_VECTOR
$EJECT
;*****************************************************************************
;
; Subroutine  DOOR_STOP
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
DOOR_STOP:
    MOV    R0,#0              ; M1=M2=0       "Off"
    CALL   BRIDGE_OFF         ; Sets motor bridge switches (direction)
    CALL   MOTOR_ENA          ; Disable motor
    CALL   MOTOR_ARM             ; Disable motor control, safety on
    CALL   MOTOR_SAFETY_OVERRIDE ; Disable Overrides Endswitches
    CALL   cACT_ENA           ; DISARMS 28V ACTUATOR PWR SWITCH
    CLR    A                  ; Zero MOTORTMR
    MOV    MOTORTMR,A         ; disable motor activity timer
    RET                       ; DOOR_STOP
$EJECT
;*****************************************************************************
;
; Subroutine  SAFE_CONTROLS
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
;*     Force controls to a known state, generally thought to be safe.
SAFE_CONTROLS:
    MOV    R0,#0              ; Turn off bits
    MOV    DPTR,#CONTROLS1+0FFh ; Select all bits of Control 1
    CLR    C1_LOCK            ; make sure Controls 1 are unlocked
    CALL   SETBIT             ; Reset Controls 1
    MOV    DPTR,#CONTROLS2+0FFh ; Select all bits of Control 2
    CLR    C1_LOCK            ; make sure Controls 2 unlocked dbb 3-21-00
    CALL   SETBIT             ; Reset Controls 2 and return
    CALL   HV_A_DAC           ; set to zero
    CALL   HV_B_DAC           ; set to zero
    CALL   HV_A_MAX           ; set to zero
    CALL   HV_B_MAX           ; set to zero
    MOV    A,#0               ; May already be reset to 0
    STOREX LFHSTATE           ; HV State always init to 0 on any reset
;
    JNB    HV_ARM_BIT,NO_DISARM ; disarm only if necessary
    CALL   HV_ARM             ; Disables HV_ON and HVQ_ON (in the software)
NO_DISARM:

; The LOCK bit was implemented incorrectly in hardware, so all accesses to the
; Controls registers will be bracketed by asserting and deasserting the LOCK
; bit.  The LOCK is set to its locked state as its default setting.
    SETB   C1_LOCK            ; P1.4 ; Controls 1 LOCK dbb 3-29-00
; per Action Item 1.028, we comment that we are initializing the three-wire
; interface to our preferred default state.  In general that is the purpose
; of this whole SAFE_CONTROLS subroutine, to force the DIB/DCE to a known state.
; While names are arbitrary, the CDC is an old FUSE term for the TDC or 
; Digitizer.  We keep the old name for no particular reason.
    CLR    CDC_B_DAV          ; P1.3 ; bit-address: data valid, CDC-B
    CLR    CDC_A_DAV          ; P1.2 ; bit-address: data valid, CDC-A
    SETB   CDC_B_DATA         ; P1.1 ; bit-address: data out, CDC-B
    SETB   CDC_B_NXT          ; P1.0 ; bit-address: clock-in, CDC-B
; These bits PHDIS will probably never be in the active state at this point,
; except possibly if a reset is triggered while reading counters and
; histograms.  But we reset them anyways, just to be extra sure.
; Even if they were wrong, the UPDATE_CTRS routine would set them right.
    CLR    GPHDIS             ; P3.3 PHDIS inactive
    CLR    CPHDIS             ; P1.5 PHDIS inactive
;
; Totally forgotten in v16200!  Fixed here in v16253, dbb 12-15-98
    CLR    HVQ_ON_BIT         ; reset indicator (RP1.15)
    CLR    HV_ARM_BIT         ; reset indicator

    CLR    ACT1_BIT           ; Actuator 1 bit (RP1.11)
    CLR    ACT2_BIT           ; Actuator 2 bit (RP1.10)

    CLR    ACT_EN_BIT         ; allows 28V actuators (RP2.9)
    CLR    AUX_PWR_ENA_BIT    ; Aux power enable bit (RP2.12)
    CLR    MOTOR_CNTRL_A_BIT  ; Motor direction control bit A (RP2.11)
    CLR    MOTOR_CNTRL_B_BIT  ; Motor direction control bit B (RP2.10)
    CLR    MOTOR_ENA_BIT      ; motor enable bit (RP1.9)

    CLR    HV_ON_BIT          ; HV on bit (RP1.16)
    CLR    HV_ARM_BIT         ; allows certain HV commands
    CLR    HVQ_ON_BIT         ; HVQ on bit
    CLR    MOTOR_ARM_BIT      ; allows motor commands

    RET                       ; SAFE_CONTROLS
$EJECT
;*****************************************************************************
;
; Subroutine  HV_I_MAX
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HV_I_MAX:                     ; HV current limit setting
    CALL   CHECKBOOT          ; check for boot mode violation
    RMOVX HV_IMAX,R0          ; Value of current limit, 0 means disable
    RET                       ; HV_I_MAX

;*****************************************************************************
DOOR_I_MAX:                   ; Door current limit setting
    CALL   CHECKBOOT          ; check for boot mode violation
    RMOVX DOOR_IMAX,R0        ; Value of current limit, 0 means disable
    RET                       ; DOOR_I_MAX
$EJECT
;*****************************************************************************
;
; Subroutine  MSG_JMPG
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v17057 dbb 07-18-00 Removed CLR CMD_ERR, changed meaning of flag
;*****************************************************************************
; If we are operating out of upper code, we jump to lower, else vice versa.
MSG_JMPG: DB 'Jumping to ',0
JMPCODE:
    CJNE   R0,#1,NOTTOLWR     ; check parameter that specifies to LCA
    SETB   CODESPACE          ; force us to assume we are in UCA
    AJMP   JUSTTOGL           ; now toggle to LCA according to CODESPACE
NOTTOLWR:
    CJNE   R0,#2,NOTTOUPR     ; check parameter that specifies to UCA
    CLR    CODESPACE          ; force us to assume we are in LCA
    AJMP   JUSTTOGL           ; now toggle to LCA according to CODESPACE
NOTTOUPR:                     ; must be 0 parameter, so we toggle
    JB     OPERATES,JUSTTOGL  ; but only if we are in operate mode
    REPORT_DIAG DIAG002F      ; Attempt to execute operate code in boot
    RET                       ; don't jump!

JUSTTOGL:
    CALL   ZAPCODECRC         ; avoid diagnostics
    MOV    DPTR,#tDP_CMD_C    ; DP_CMD_C
    CALL   INCREMENT_WORD     ; DPU command counter
    CLR    A                  ; INCREMENT_WORD assigns ACC, now zeroed.
    MOV    LEDimage,A         ; Zap image: lower image may not reset bits
    MOV    R0,A               ; low goto pointer
    MOV    R2,#0B0h           ; high goto pointer
    JB     CODESpace,JMPDN    ; use code space bit to determine where to go
    MOV    A,#40h             ; This is the address bit to turn on: B0 -> F0
    ORL    A,R2               ; turn on the bit
    MOV    R2,A               ; modified for upper
JMPDN:
;    CLR    CMD_ERR            ; avoid Detect command fail
    JMP    GOTO               ; JMPCODE
$EJECT
;*****************************************************************************
;
; Subroutine  SAVE_CRC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAVE_CRC:
    MOV    A,CRCL
    STOREX tSAVE_CRC          ; save CRC in an otherwise undefined slot 28 15
    MOV    A,CRCH
    STORNEXT
    RET                       ; SAVE_CRC
$EJECT
;*****************************************************************************
;
; Subroutine  UPLOAD
;
; INPUTS -- Command parameters in registers R2:R0 start addr, R3:R1 length
;           other parameters taken from CMDIMAGE buffer.
; OUTPUTS -- Overwrites most registers
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v17047 dbb 07-06-00 Comments changed per Action Item 1.053
; 
;*****************************************************************************
UPLOAD:                       ; This entry is for the upload command.
$IF XXX			      ; not tested yet, left out, dbb 7-6-00
; If sent CRC is zero, we don't require CRCs to match before copying upload
; data to executable memory.  This option degrades the original design intent
; to ensure that code uploads are intact before overwriting memory.  In
; operations, the CRC should never be sent as 0.  Per Action Item 1.054
    FETCHX RS17_PAR2          ; point to parameter holding CRC, low part first
    CJNE   A,#0,B8D_CRC       ; verify low part matches calculated value
    FETCHNEXT                 ; fetch the high part of CRC
    CJNE   A,#0,B8D_CRC       ; verify hi part matches calculated value
    SETB   F0                 ; we check CRC before actual move!
B8D_CRC:
$ELSE
    SETB   F0                 ; we check CRC before actual move!
$ENDIF
    PRESETCRC                 ; preset CRC to FFFF
                ; On entry, startaddr is in R2:R0, endaddr in R3:R1, and the
                ;   CRC is still in the commmand buffer, parameter 3.
    MOV    A,R2      ; high-byte of start addr -- put in correct registers
    MOV    R5,A      ; high-byte of start addr --  for call to BYTE_MOVE
    MOV    A,R0      ; low-byte of start addr
    MOV    R4,A      ; low-byte of start addr

    MOV    A,R3       ; Look at high-byte of Endaddr, interpreted as length
    CPL    A          ; 1's complement so we can use adder to check magnitude
    ADD    A,#4h      ; compare length to 1023 by setting carry flag
    JC     ENDADDROK  ; skip upload if Endaddr == length > 3FFh == 1023
    MOV    A,R3       ; It looks like length too long, but 1024 is okay
    CJNE   A,#4,BAD_LENGTH ; So R3 must be 4, and
    MOV    A,R1       ; R1 (the low-byte of length) must be zero
    JZ     ENDADDROK  ;  and if r3:r1 is 1024, then it is okay
BAD_LENGTH:
    REPORT_DIAG DIAG0001      ; Diagnostic code for bad length
    JMP    UPL_COMPLETE       ; single exit
BAD_CRC:
    REPORT_DIAG DIAG0002      ; Diagnostic code for bad CRC
    CALL   SAVE_CRC           ; put CRC value in housekeeping
    AJMP   UPL_COMPLETE       ; single exit
ENDADDROK:
; put the length somewhere for use below... R3 needed below for source ptr
    MOV    A,R3               ; high-byte length
    MOV    R7,A               ;  saved somewhere safe, low-byte still in R1
    MOV    R2,#LOW(RS01)      ; low-byte ptr to source
    MOV    R3,#HIGH(RS01)     ; high-byte ptr to source
;
    INC    R7             ; actual total number of calls to BYTE_MOVE required
    MOV    A,R1       ; low-byte length needs to be in R0 for BYTE_MOVE
    MOV    R0,A       ; so we put it there
    JNZ    MOVE_LOOP  ; but we only use it if non-zero, or...
                      ;   or if high-byte length is non-zero
    DJNZ   R7,MOVE_LOOP       ;   then R0==0 means move 256 bytes
    JMP    BAD_LENGTH         ; otherwise, a length of 0 is not valid
;
MOVE_LOOP:
; Looking through memory will change all the registers, so we save them.
    JB     F0,SAVE_REGS       ; if we are checking CRC, then save registers
    AJMP   MOVE_AGAIN         ; otherwise we won't be popping them
SAVE_REGS:
    PUSH   0                  ; saves R0 during CRC checking phase of upload
    PUSH   2                  ; saves R2 during CRC checking phase of upload
    PUSH   3                  ; saves R3 during CRC checking phase of upload
    PUSH   4                  ; saves R4 during CRC checking phase of upload
    PUSH   5                  ; saves R5 during CRC checking phase of upload
    PUSH   7                  ; saves R7 during CRC checking phase of upload
    CLR    SPECIALF           ; be sure to read from RAM
MOVE_AGAIN:
    CALL   BYTE_MOVE          ; on return R5:R4 and R3:R2 point to next move
    DJNZ   R7,MOVE_AGAIN      ; and move another 256 bytes as required
    JBC    F0,CHECK_CRC       ; if we are checking CRC, then verify and move
    AJMP   UPL_COMPLETE       ; otherwise, we are done already

CHECK_CRC:
    FETCHX RS17_PAR2          ; point to parameter holding CRC, low part first
    CJNE   A,CRCL,B0D_CRC     ; verify low part matches calculated value
    FETCHNEXT                    ; fetch the high part of CRC
    CJNE   A,CRCH,B0D_CRC     ; verify hi part matches calculated value
; This guarantees that we pop everything that we push, right?
    JMP    B1D_CRC
B0D_CRC:
    SETB   F0                 ; flag CRC error
B1D_CRC:
    POP    7                      ; saves R7 during CRC checking phase of upload
    POP    5                  ; saves R5 during CRC checking phase of upload
    POP    4                  ; saves R4 during CRC checking phase of upload
    POP    3                  ; saves R3 during CRC checking phase of upload
    POP    2                  ; saves R2 during CRC checking phase of upload
    POP    0                      ; saves R0 during CRC checking phase of upload
    JBC    F0,B2D_CRC         ; BAD_CRC flag?
    CALL   SAVE_CRC           ; put CRC value in housekeeping
    AJMP   MOVE_AGAIN         ; This time, move the data
B2D_CRC:
    JMP    BAD_CRC            ; BAD_CRC flag?

UPL_COMPLETE:
    RET                       ; lUPLOAD
$EJECT
;*****************************************************************************
;
; Subroutine  DUMP
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; Tested! dbb 9-13-96
DUMP:           ; Dumps memory block: Startaddr, Length
                ; On entry, startaddr is in R2:R0, endaddr in R3:R1, and the
                ;   CRC is still in the commmand buffer, parameter 3.

                ; Now, this doesn't really make sense: the Endaddr must be
                ;   constrained to a value larger than StartAddr and less than
                ;   64 bytes larger than StartAddr.  I think I will ignore
                ;   Endaddr unless it is a value 1..1024, in which case I treat
                ;   it as a length byte.
    MOV    A,R3       ; Look at high-byte of Endaddr, interpreted as length
    CPL    A          ; 1's complement so we can use adder to check magnitude
    ADD    A,#4h      ; compare length to 1023 by setting carry flag
    JC     XENDADDROK  ; skip upload if Endaddr == length > 3FFh == 1023
    MOV    A,R3       ; It looks like length too long, but 1024 is okay
    CJNE   A,#4,DBAD_LENGTH ; So R3 must be 4, and
    MOV    A,R1       ; R1 (the low-byte of length) must be zero
    JZ     XENDADDROK  ;  and if r3:r1 is 1024, then it is okay
DBAD_LENGTH:
    REPORT_DIAG DIAG0003   ; Diagnostic code for bad length
    JMP    DUPL_COMPLETE      ; single exit
DENDADDROK:
; Looking through memory will change all the registers, so we save them.
    POP    7                      ; hi length
    POP    5                  ; hi dest
    POP    4                  ; lo dest
    POP    3                  ; hi source
    POP    2                  ; lo source
    POP    0                      ; lo length
XENDADDROK:
; put the length somewhere for use below... R3 needed below for source ptr
    MOV    A,R3               ; high-byte length
    MOV    R7,A               ;  saved somewhere safe, low-byte still in R1
; now we set up the source ptr this way just to make it clear why R3:R2 == 0
    MOV    A,R2      ; high-byte of start addr -- put in correct registers
    MOV    R3,A      ; high-byte of start addr --  for call to BYTE_MOVE
    MOV    A,R0      ; low-byte of start addr
    MOV    R2,A      ; low-byte of start addr
; now set up dest
    MOV    DPTR,#TS01         ; TS01 is the destination of dumped data
    MOV    R4,DPL             ; low-byte ptr to dest
    MOV    R5,DPH             ; high-byte ptr to dest
;
    INC    R7             ; actual total number of calls to BYTE_MOVE required
    MOV    A,R1       ; low-byte length needs to be in R0 for BYTE_MOVE
    MOV    R0,A       ; so we put it there
    JNZ    DMOVE_LOOP  ; but we only use it if non-zero, or...
                      ;   or if high-byte length is non-zero
    DJNZ   R7,DMOVE_LOOP       ;   then R0==0 means move 256 bytes
    AJMP   DBAD_LENGTH         ; otherwise, a length of 0 is not valid
;
DMOVE_LOOP:
    LCALL  BYTE_MOVE          ; on return R5:R4 and R3:R2 point to next move
    DJNZ   R7,DMOVE_LOOP      ; and move another 256 bytes as required
;
; No CRC calc!  dbb 11-22-96
    SETB   F0                 ; we check CRC after actual move!
    JB     F0,SAVEtCRC        ; exit after calculating CRC
    PRESETCRC                 ; preset CRC to FFFF
    PUSH   0                      ; lo length
    PUSH   2                  ; lo source ptr
    PUSH   3                  ; hi source ptr
    PUSH   4                  ; lo dest ptr
    PUSH   5                  ; hi dest ptr
    PUSH   7                     ; hi length
    JMP    DENDADDROK         ; now pass through again, calculate CRC
SAVEtCRC:
; Save the CRC as calculated
    CLR    F0                 ; restore default state of this bit
    CALL   SAVE_CRC           ; put CRC value in housekeeping
DUPL_COMPLETE:
    CALL   DMPPACKET          ; send dump packet
    RET                       ; DUMP
$EJECT
;*****************************************************************************
;
; Subroutine  CHECK_COMMAND
;
; INPUTS -- none
; OUTPUTS -- ACC, DPTR changed
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v17062 dbb 07-22-00 Added handshake for GF0 disable BACKGROUND after NOOP
;
;*****************************************************************************
CHECK_COMMAND:
    FETCHX CMDIMAGE           ; load address of command buffer
    JNZ    LIVEONE            ; when 0, we assume commands not required.
    CLR    GF0                ; restore BACKGROUND activity after this routine
    JMP    FEED_DOG           ; prevent reset when command buffer untouched
    RET                       ; CHECK_COMMAND
    NOP                       ; word alignment

LIVEONE:
    CJNE   A,#NULL_CMD,SOME_COMMAND ; Is there a command to process?
    JMP    NEXT_THING         ; RET asap, one exit point preferred
SOME_COMMAND:
; we want to be sure the packet counter is incremented first
    PUSHDPTR
    PUSH   ACC                ; save command byte
    MOV    DPTR,#TS28         ; Packet counter
    CALL   INCREMENT_DWORD
    POP    ACC                ; restore command byte
    POPDPTR

; Diagnostics...
; v17057 dbb 07-18-00 Changed meaning of flag
    SETB   CMD_ERR            ; Detect command fail

; Now we do some validation on the contents of RS17
    RLC    A                  ; put MSB in C
    JC     GOOD_1_CMD         ; MSB must be 1
    REPORT_DIAG DIAG0004      ; Wrong command MSB
    JMP    NEXT_THING         ; RET asap, one exit point preferred

GOOD_1_CMD:
    MOVX   A,@DPTR            ; fetch command
    MOV    R0,A               ; save the command byte for comparison to twin
    FETCHNEXT                     ; fetch command twin
    XRL    A,R0               ; compare the twin commands, should be zero
    JZ     GOOD_2_CMD         ; twin commands must be the same
    REPORT_DIAG DIAG0005      ; Command byte must have twin
    JMP    NEXT_THING         ; RET asap, one exit point preferred

GOOD_2_CMD:
    MOV    DPTR,#CMDIMAGE-1   ; load address of command buffer
    MOV    R1,#05H            ; we examine the rest of the parameters only
CHECK_NEXT_PARAM:
    FETCHNEXT                     ; point to parameter
    MOV    R0,A               ; save low-byte for later comparison
    INC    DPTR               ; skip high-byte
    MOV    R2,DPL             ;   but we will want to come back, so save it
    MOV    R3,DPH             ;   but we will want to come back, so save it
    FETCHNEXT                     ; point to low-byte complement
    XRL    A,R0               ; compare low-byte parameter with complement
    CPL    A                  ; should be 0FFh; make it 0 for test
    JZ     GOOD_LO_PARAMS     ; and decide if complement was valid
    REPORT_DIAG DIAG0006      ; Parameters must have complements
    JMP    NEXT_THING         ; RET asap, one exit point preferred
GOOD_LO_PARAMS:
    MOV    DPL,R2             ;   restore pointer to high-byte
    MOV    DPH,R3             ;   restore pointer to high-byte
    MOVX   A,@DPTR            ; fetch parameter high-byte
    MOV    R0,A               ; save high-byte for later comparison
    INC    DPTR               ; skip low-byte complement
    FETCHNEXT                     ; fetch parameter high-byte complement
    XRL    A,R0               ; compare high-byte parameter with complement
    CPL    A                  ; should be 0FFh; make it 0 for test
    JZ     GOOD_HI_PARAMS     ; and decide if complement was valid
    REPORT_DIAG DIAG0006      ; Command Parameters must have complements
    AJMP   NEXT_THING         ; RET asap, one exit point preferred
GOOD_HI_PARAMS:
    DJNZ   R1,CHECK_NEXT_PARAM    ; do all the parameters

; Most commands take 0,1,or 2 parameters.  We will always
;   fetch the first two parameters and put them in R0 and R1.  If additional
;   parameters are required, then the routine involved with fetch them.
;   High-bytes in R2 and R3 for parameters 0 and 1, respectively.

    FETCHX RS17_PAR0          ; load address of command parameter 0
    MOV    R0,A               ; pass parameter via register
    FETCHNEXT                 ; fetch command high-byte parameter 0
    MOV    R2,A               ; pass high-byte parameter via register
    FETCHX RS17_PAR1          ; load address of command parameter 1
    MOV    R1,A               ; pass parameter via register
    FETCHNEXT                 ; fetch command high-byte parameter 1 
    MOV    R3,A               ; pass high-byte parameter via register

; Reload the command and call the associated routine
    FETCHX CMDIMAGE           ; load address of command buffer 
    CALL   COMMAND_VECTOR     ; process the command now located in A
;
    FETCHX CMDIMAGE           ; CMD
    ANL    A,#7Fh             ; check for NO_OPER
    JZ     CMD_COMPLETE       ; don't count NO_OPERs 11-14-96
    MOVX   A,@DPTR            ;   fetch it again, since we mutilated the last
    STOREX tCMD_R             ; CMD_R to...

    MOV    DPTR,#tDP_CMD_C    ; DP_CMD_C
    CALL   INCREMENT_WORD     ; DPU command counter
CMD_COMPLETE:
    TOGLEDS 4                 ; Command executed!
    CLR    CMD_ERR            ; Detect command fail
    CALL   FEED_DOG           ; prevent reset when controlling via MON51
NEXT_THING:
    CLR    GF0                ; restore BACKGROUND activity after this routine
    MOV    A,#NULL_CMD        ; Use to indicate no_command, next time
    STOREX CMDIMAGE           ; Ack command by writing over it
    RET                       ; CHECK_COMMAND
$EJECT
;*****************************************************************************
;
; Subroutine  INCREMENT_WORD
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
INCREMENT_WORD:               ; address of word to increment in DPTR
    CLR    C                  ; assume no carry on first add of LSB
    MOVX   A,@DPTR            ; fetch the low byte of word to be incremented
    ADDC   A,#1               ; increment means add 1, set carry flag
    MOVX   @DPTR,A            ; update the low byte of word
    FETCHNEXT                 ; fetch the high byte of word
    ADDC   A,#0		      ; add carry flag C if necessary
    MOVX   @DPTR,A	      ; update the high byte of word
    RET                       ; INCREMENT_WORD changes A,DPTR
$EJECT
;*****************************************************************************
;
; Subroutine  INCREMENT_DWORD
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; The packet counter is four bytes wide.  Here we increment it.
INCREMENT_DWORD:
    MOV    R0,#4              ; max number of bytes in counter
    SETB   C                  ; assume carry to force increment on first
NEXT_BYTE:                    ;  time thru loop
    MOVX   A,@DPTR            ; fetch a byte from the packet counter
    ADDC   A,#0               ; count one packet *duh!*
    MOVX   @DPTR,A            ; and put it back
    JNC    INC_COMPLETE       ; if carry, then that byte rolled over,
    INC    DPTR               ; must be a carry, so look at next higher byte
    DJNZ   R0,NEXT_BYTE       ; limit the precision of the counter...
INC_COMPLETE:
    RET                       ; INCREMENT_DWORD
$EJECT
;*****************************************************************************
;
; Subroutine  COUNT_CMDS_RXD
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;***************************************************************************
COUNT_CMDS_RXD:               ; update 28/13 DP_CMD_R
    JZ     CCR_EXIT           ; test ACC, if 0 cmd was NO_OPER -- don't count!
    PUSH   ACC                ; save the offset into command table
    MOV    DPTR,#tDP_CMD_R    ; DP_CMD_R
    CALL   INCREMENT_WORD     ; DPU commands received
$IF XSPEC
; a debug command activity counter is implemented here to make it easier for
; lab users to know if someone else is using the testbed.  Zeroed on command.
    ASSIGNX tU7,0             ; used for commands timer
    ASSIGNX tU2,0             ; used for commands timer
$ELSE
    SPARE_CODE 20             ; account for xspec code left out of flight
$ENDIF
    POP    ACC                ; restore the offset into command table
CCR_EXIT:
    RET                       ; COUNT_CMDS_RXD
$EJECT
;*****************************************************************************
;
; Subroutine  HKPFULL
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HKPFULL:                      ; force next hkp packet to contain 512 dwords
    SETB   HKP_FULL           ; set indicator, reset in mainroutine
    RET                       ; HKPFULL next hkp packet to contain 512 dwords
$EJECT
;*****************************************************************************
;
; Subroutine  ; The region to use depends on the DATA/CODE source of the CRC measurement
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; The region to use depends on the DATA/CODE source of the CRC measurement:
;   REGION0 if DATA; REGION1 if CODE.  We fill in only the address and
;   length part of the region info, leaving the rest as initialized.
; v17009 dbb 5-28-00 fixed some typos... DL-->DPL, etc
CALC_CRC:
    FETCHX RS17_PAR2          ; get MEMSPACE parameter, 0 = DATA, 1 = CODE
    JZ     NOTCODE            ; 0 = DATA, non-zero taken as Code
    MOV    DPTR,#REGION1      ; look at codespace, as REGION1 is setup to do
    AJMP   MODREGION          ;  continue after correct pointer loaded DPTR
NOTCODE:
    MOV    DPTR,#REGION0      ; look at dataspace, as REGION0 is setup to do
MODREGION:
    MOV    CONTEXT,DPL	      ; save the pointer to the region in internal
    MOV    CONTEXT+1,DPH      ;   memory, DPTR retrieved with LOAD_CONTEXT
    MOV    A,R0               ; low part of Address
    MOVX   @DPTR,A            ; save Address in REGION info
    MOV    A,R1               ; high part of Address
    STORNEXT                  ; save Address in REGION info
    CLR    C                    ; no carry in ADDC below
    MOV    A,R0               ; low part of Address
    ADDC   A,R1               ; low part of Length
    STORNEXT                  ; save End-Address in REGION info
    MOV    A,R2               ; high part of Address
    ADDC   A,R3               ; high part of Length, with carry
    STORNEXT                  ; save End-Address in REGION info

    JMP    INIT_REGION        ; CALC_CRC

$IF XXX
;*****************************************************************************
TEST_TABLE:
    DW     14, TS25+24h, RS25
    DW     0,0,0
$ENDIF
$EJECT
;*****************************************************************************
;
; Subroutine  LCOPY
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
LCOPY:
    MOV    A,R7               ; check high byte of length
    INC    ACC                ; upper byte of length is incremented for DJNZ
    MOV    R7,A               ; upper byte of adjusted length for loop control
    INC    DPTR               ; point to next table-item
    CLR    F0                 ; F0 selects CRC instead of actual move
    PUSHDPTR                  ; save pointer to next table-item
ONCEAGAIN:                    ;  Loop copying blocks of no more than 256 bytes
    CALL   BYTE_MOVE          ; on return R5:R4 and R3:R2 point to next move
    DJNZ   R7,ONCEAGAIN       ; and move another 256 bytes as required
    POPDPTR		      ; restore pointer to next table-item
    RET                       ; LCOPY
$EJECT
;*****************************************************************************
;
; Subroutine  ; specifier table is zero.  Each table entry is 3 words
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; Performs a series of copy functions, using the table at DPTR for source and
; destination.  The table is terminated when the length part of the copy
; specifier table is zero.  Each table entry is 3 words: length, source, dest.
TABLE_COPY:
    CLR    A                  ; zero
    MOVC   A,@A+DPTR          ; fetch length specifier from table data
    MOV    R7,A               ; length high-byte comes first
    INC    DPTR               ; point to next table-item
    CLR    A                  ; zero
    MOVC   A,@A+DPTR          ; fetch length specifier from table data
    MOV    R0,A               ; then length low-byte
    INC    DPTR               ; point to next table-item
    CLR    A                  ; zero
    MOVC   A,@A+DPTR          ; fetch table data from code space
    MOV    R3,A               ; source high-byte comes first
    INC    DPTR               ; point to next table-item
    CLR    A                  ; zero
    MOVC   A,@A+DPTR          ; fetch table data from code space
    MOV    R2,A               ; then source low-byte
    INC    DPTR               ; point to next table-item
    CLR    A                  ; zero
    MOVC   A,@A+DPTR          ; fetch table data from code space
    MOV    R5,A               ; dest high-byte comes first
    INC    DPTR               ; point to next table-item
    CLR    A                  ; zero
    MOVC   A,@A+DPTR          ; fetch table data from code space
    MOV    R4,A               ; then dest low-byte
; before we process the copy, we check for length of zero, which means stop
; copying table-items.
    MOV    A,R0               ; check low byte of length
    JNZ    NOTTHEEND          ; if lower byte is zero, this could be the end
    MOV    A,R7               ; check high byte of length
    JNZ    NOTTHEEND          ; upper byte zero, too, so we're done with table.
    RET                       ; TABLE_COPY

NOTTHEEND:
    CALL   LCOPY              ; R7:R0, R3:R2, R5:R4, SPECIALF
    AJMP   TABLE_COPY         ; look at next table item.
$EJECT
;*****************************************************************************
;
; Subroutine  MEM_COPY
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
MEM_COPY:                     ; copy memory-to-memory on command
    FETCHX RS17_PAR0          ; low byte is stored first, source pointer
    MOV    R2,A               ; source low-byte
    FETCHNEXT                 ;  get high-byte
    MOV    R3,A               ; source high-byte
    FETCHX RS17_PAR1          ; low byte is stored first, dest pointer
    MOV    R4,A               ; dest low-byte
    FETCHNEXT                    ;  get high-byte
    MOV    R5,A               ; dest high-byte
    FETCHX RS17_PAR2          ; low byte is stored first, length
    MOV    R0,A               ; length low-byte
    FETCHNEXT                    ;  get high-byte
    MOV    R7,A               ; length high-byte
    FETCHX RS17_PAR3          ; low byte is stored first, length
    CJNE   A,#1,FROMDATA      ; MEMSPACE sets SPECIALF, data comes from code
    SETB   SPECIALF           ; let next BYTE_MOVE read from codespace
FROMDATA:
    CALL   LCOPY              ; R7:R0, R3:R2, R56:R4
    CLR    SPECIALF           ; we prefer to leave it in this state
    RET                       ; MEM_COPY
$EJECT
;*****************************************************************************
;
; Subroutine  FETCHPTR
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; DPTR to the base of the structure, A is index of words
FETCHPTR:                     ; use DPTR to load DPTR
    PUSH   B
; Adapted for use with commands, dbb 6-27-00    ANL    A,#1Fh             ; max list length is 16 items, enforced here
                              ; index of bytes --> words --> --> dwords
    RL     A                  ; now A is index to item data
    PUSH   ACC                ; save index on stack
    MOVC   A,@A+DPTR          ; fetch high byte
    MOV    B,A                    ; stash high byte pointer
    POP    ACC                    ; save index on stack
    INC    ACC                    ; point index to next byte
    MOVC   A,@A+DPTR          ; fetch high byte
    MOV    DPL,A              ; load low byte of pointer
    MOV    DPH,B              ; load high byte of dptr with stashed code
    POP    B
    RET                       ; FETCHPTR           ; use DPTR to load DPTR
$EJECT
;*****************************************************************************
;
; Subroutine  REPORT_DIAGNOSTIC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; Modified to support HST-errors, which are different from diagnostics in
;   that they are appended to the error-list, which is cleared after the
;   hkp-packet is sent.
REPORT_DIAGNOSTIC:
    PUSH   0
    PUSH   1
    SETB   GF0                ; suppress BACKGROUND activity for this routine
    PUSH   ACC                ; save it!
    MOV    R0,DPL             ; Diagnostic was passed in DPTR, needed below
    MOV    R1,DPH             ; Diagnostic was passed in DPTR, needed below
    MOV    A,R0                    ;  A holds item to store in tDIAG0
    STOREX tDIAG0             ; CSTAT put diagnostic code in CSTAT

; process HST error:
    PUSH   ACC                ; save it! because ACC is used in search below
    MOV    R0,#8              ; max number of hst errors allowed per second
    MOV    R1,#0             ; zero count of index to open table location
    FETCHX tLFDERR            ; search for a place to put the diag code
LOOKAGIN:
    JZ     PUTITHERE          ; space available when zero
    FETCHNEXT                 ; else we go on to the next location in list
    INC    R1                 ; keep count of index to open table location
    DJNZ   R0,LOOKAGIN        ; keep looking until we hit the end of list
PUTITHERE:
    POP    ACC                    ; restore it! now we recover the diag
    MOVX   @DPTR,A            ; place diag in hst error-list
;
    PUSH   ACC                ; save it! because ACC is used in search below
;
    MOV    DPTR,#HSTERR_TABLE ; values pointed-to
    CALL   FETCHPTR           ; load dptr with value pointed to by (A*2)+dptr
    CLR    A                  ; zero index
    CALL   FETCHPTR           ; load dptr with value pointed to by (A*2)+dptr
    MOV    B,DPL              ; the lsb of pointed-to value
    MOV    A,DPH             ; the msb of pointed-to value
    STOREX tLFDERRP           ; point to parameter space
    MOV    A,B
    STORNEXT

    POP    ACC                     ; restore it! now we recover the diag

    MOV    R0,A               ; save low byte, diag code
; Move the list of diagnostics down one word; 32nd diagnostic is lost.
    PUSH   0                  ; save R0
    PUSH   1                  ; save R1
    MOV    DPTR,#TS30+64      ; point past end of TS30
    MOV    R0,#62             ; number of bytes to move
dLOOP:
    DEC    DPL                ; point to destination
    PUSH   DPL                ; save pointer to dest
    DEC    DPL                ; point low...
    DEC    DPL                ; point low...
    MOVX   A,@DPTR            ; fetch from a lower address...
    POP    DPL                    ; point high...
    MOVX   @DPTR,A            ; write to a higher address
    DJNZ   R0,dLOOP           ; do each byte...
    POP    1                  ; restore R1
    POP    0                  ; restore R0
;
    MOV    A,R0               ; prepare low part of Diagnostic code
    STOREX TS30               ; put low part in TS30 space
    INC    DPTR               ; point to upper part
; upper byte of diagcode not used... dbb 2-2-00
;    MOV    A,R1               ; prepare high part
    MOVX   A,@DPTR            ; get the previous serial number
    INC    A                  ; advance serial number
    MOVX   @DPTR,A            ; put serial number in TS30 space

; Patch telemetry for new layout 4-3-97
    FETCHX TS30
    STOREX tDIAG0             ; 28 5 DIAG0
    FETCHX TS30+02h
    STOREX tDIAG1             ; 28 5+ DIAG1
    FETCHX TS30+04h
    STOREX tDIAG2             ; 28 9+ DIAG2
    FETCHX TS30+06h
    STOREX tDIAG3             ; 28 11+ DIAG3
    FETCHX TS30+08h
    STOREX tDIAG4             ; 28 13+ DIAG4

    CLR    GF0                ; restore BACKGROUND activity after this routine
    POP    ACC                    ; restore it!
    POP    1                  ; restore R1
    POP    0                  ; restore R0
    RET                            ; REPORT_DIAGNOSTIC
$EJECT
;*****************************************************************************
;
; Subroutine  ZAPWORD
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
ZAPWORD:                      ; make zero the word pointed to by DPTR
    CLR    A                    ; zero
    MOVX   @DPTR,A            ; make zero the word pointed to by DPTR
    STORNEXT
    RET                       ; ZAPWORD
$EJECT
;*****************************************************************************
;
; Subroutine  ZAPCODECRC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; We are trying to avoid diagnostic 18, generated because some v162 codespace
; is being used to store sample timestamps used for overlimit checking
ZAPCODECRC:
    MOV    DPTR,#Image2       ; lower code area CRC value
    CALL   ZAPWORD            ; zero out CRC value to avoid triggering report
    MOV    DPTR,#Image3       ; upper code area CRC value
    CALL   ZAPWORD            ; zero out CRC value to avoid triggering report
    RET                       ; ZAPCODECRC
$EJECT
;*****************************************************************************
;
; Subroutine  SETMUX
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SETMUX:                       ; set according to param1
    MOV    R2,#LOW(tPOINTMON)
    MOV    R3,#HIGH(tPOINTMON)
    MOV    DPH,#HIGH(MUX)
    FETCHX RS17_PAR0          ; MUX selector in param 0
    MOV    DPL,A
    JMP    CONVERT_A2D        ; expects DPTR and R3:R2
    RET                       ; SETMUX
$EJECT
;*****************************************************************************
;
; Subroutine  ;  Inputs
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
$IF XSPEC
; This is a backdoor command for sending direct commands to the TDC.
; Normally used for debugging, and out of spec for COS.  Works only for
; the A-side TDC.
;  Inputs: R1 and R0 hold command and selector, respectively
;  Outputs: A, R0 destroyed by CDC_A_OUT
TDC:
    PUSH   1
    PUSH   0
    POP    ACC
    POP    0
    SETB   MUX_FLAG           ; required to disable a lookup function
    CALL   CDC_A_OUT          ; send selector and command to CDC_A only
    CLR    MUX_FLAG           ; required to enable a lookup function
    RET                       ; tTDC
$ELSE
    SPARE_CODE 16             ; account for xspec code left out of flight
$ENDIF
$EJECT
;*****************************************************************************
;
; Subroutine  HKPTEST
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HKPTEST:
; These timestamps are used to support a test to verify that housekeeping
; packets are delivered intact and contemporaneously  dbb 3-29-00
;  Use of this code overwrites portions of the housekeeping packet
    MOV_TIMER ENDPACKETTIMESTAMP ; Save timer value - hkpupdate
    MOV_TIMER STAPACKETTIMESTAMP ; Save timer value - hkpupdate dbb debug 4-1-00
    RET                          ; HKPTEST

$IF DEGAP
    FROGS_CODE  776           ; opcode 0xFF virtual nops fill extra space
    FROGS_CODE 14
    TRAP_CODE     1           ; DIAGNOST SEGMENT CODE UNIT
$ENDIF
$EJECT
;*****************************************************************************
;
; Subroutine  CHECKBOOT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; v16876 dbb 05-02-00 Boot State Command Rejection Implemented
;*****************************************************************************
; Returns nothing if code is in Operate Mode, else issues a diagnostic and
; jumps to the reentry vector (an unstructured exit)
CHECKBOOT:
    JB     OPERATES,NOTBOOT
    REPORT_DIAG DIAG002F      ; BOOT MODE COMMAND VIOLATION
    JMP    REENTRY            ; an unstructured exit, command not executed
NOTBOOT:
    RET                       ; CHECKBOOT
$EJECT
;*****************************************************************************
;
; Subroutine  ;  Inputs
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; The Counter Actels may require an occasional commanded reset.  The FPGAs
; are normally reset only on POR.  The timing requirement is unclear, so the
; looping and calls to background are guesses intended to provide a wide reset
; pulse to the Counter Actels.
;  Inputs: none
;  Outputs: R1 destroyed, CMD_RESET set inactive
ACTEL_RESET:                  ; 7AH  Toggle reset line to counter actel
    CLR    CMD_RESET          ; active Actel reset, Counters not functioning
    CALL   BACKGROUND         ; compute crcs to kill time, long reset pulse
                              ; R1 is probably zero the first time thru,
                              ;  so we will probably loop 256 times.  But,
    DJNZ   R1,ACTEL_RESET     ; we don't really care how many times we loop...
    SETB   CMD_RESET          ; inactive Actel reset, allows Counters function
    RET                           ; ACTEL_RESET
$EJECT
;*****************************************************************************
;
; Subroutine  ; R0 holds the command parameter which should be one of the following
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; R0 holds the command parameter which should be one of the following:
;  1 - Nom A  (B Low)
;  2 - Nom B  (A Low)
;  3 - Nom AB (Both Nom)
;  4 - HV Low (Both Low)
; Editorial view: This specification originates with the documented command
; described in DM-05.  It was intended to simplify HV commmanding but
; simply provides another way to set HV.
;
; There is no error checking on the parameters, allowing for back-compatiblity
; with the legacy FUSE design.  Parameter values out of bounds will be wrapped
; to one of the valid values above.
HSTATE:
    MOV    A,R0               ; fetch parameter
    ANL    A,#03h             ; Bounds wrapped
    RL     A                  ; multiply by two for instruction size
    MOV    DPTR,#HST_TBL      ; Use jump table below to select command
    JMP    @A+DPTR
HST_TBL:
    AJMP   HST4               ; 0 HV Low, Both Low
    AJMP   HST1               ; 1 HV NOM A, B Low
    AJMP   HST2               ; 2 HV NOM B, A Low
    AJMP   HST3               ; 3 HV NOM, Both NOM
HSTX:
    STOREX LFHSTATE           ; HV State always init to 0 on any reset
    RET                       ; HSTATE
SETNOMA:
    CLR    CTRS_FLAG          ; make sure ramp happens only on command
    SETB   SA_STATE           ; change A side
    FETCHX LFHVNOMA           ; fetch saved target
    STOREX SA_SAVED           ; put target hv setting in sx_saved variable
    RET                       ; SETNOMA
SETNOMB:
    CLR    PHS_FLAG           ; make sure ramp happens only on command
    SETB   SB_STATE	      ;  and change B side
    FETCHX LFHVNOMB           ; fetch saved target
    STOREX SB_SAVED           ; put target hv setting in sx_saved variable
    RET                       ; SETNOMB
SETLOWA:
    CLR    CTRS_FLAG          ; make sure ramp happens only on command
;    CPL    SA_STATE           ; change A side
    FETCHX tSAA_HV_A          ; fetch saved target
    STOREX SA_SAVED           ; put target hv setting in sx_saved variable
    RET                       ; SETLOWA
SETLOWB:
    CLR    PHS_FLAG           ; make sure ramp happens only on command
;    CPL    SB_STATE	      ;  and change B side
    FETCHX tSAA_HV_B          ; fetch saved target
    STOREX SB_SAVED           ; put target hv setting in sx_saved variable
    RET                       ; SETLOWB
HST1:                         ; Process 1 HV NOM A, B Low
    CALL   SETNOMA            ; load resume value with nominal setting
    CALL   SETLOWB            ; load resume value with low setting
    CALL   SAA_OVER           ; 6EH *day - Restore HV saved during shutdown
    MOV    A,#1               ; Event D
    JMP    HSTX               ; All done, HV state update
HST2:                         ; Process 2 HV NOM B, A Low
    CALL   SETLOWA            ; load resume value with low setting
    CALL   SETNOMB            ; load resume value with nominal setting
    CALL   SAA_OVER           ; 6EH *day - Restore HV saved during shutdown
    MOV    A,#2               ; Event E
    JMP    HSTX               ; All done, HV state update
HST3:                         ; Process 3 HV NOM AB
    CALL   SETNOMA            ; load resume value with nominal setting
    CALL   SETNOMB            ; load resume value with nominal setting
    CALL   SAA_OVER           ; 6EH *day - Restore HV saved during shutdown
    MOV    A,#3               ; Event F
    JMP    HSTX               ; All done, HV state update
HST4:                         ; Process 4 HV Low, Both Low
    CALL   SAA_HERE           ; 6FH *night - Trigger shutdown, no auto restore
    MOV    A,#4               ; Event G
    JMP    HSTX               ; All done, HV state update

;*****************************************************************************
NOMSTATE:
    JMP    HST3               ; Nom AB
    RET                       ; NOMSTATE
$EJECT
;*****************************************************************************
;
; Subroutine  JNTTEST
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; This is the interface to the J&T Test Code.  The F0 flag is a hook to allow
; Tim Swanson (J&T) to call his code repeatedly, dumping output, while the
; "Do Test" Flag (F0) is set.  The JNTCODE controls F0.
; JNTTEST Command parameters are in the command buffer CMDIMAGE
;   param0 at 166c; param1 at 1670 ; param2 at 1674 ; param3 at 1678
;   param4 at 167c
JNTTEST:
    CLR    F0                 ; flag used to indicate okay to copy xfer data
    CALL   JNTCODE            ; Goodbye!
    CALL   DMPPACKET          ; send dump packet
    JB     F0,JNTTEST         ; loop until F0 flag is returned clear
    RET                       ; JNTTEST

    END                       ; COMMANDS.A51
