#!/opt/misc/bin/perl
# #!/bin/sh
                                  $hkgroup1::version = "hks v1.009 14 Apr 00";
#-----------------------------------------------------------------------------
#
#                    GGGGG                                          1
#  H    H   K    K  G     G   RRRRR     OOOO    U    U   PPPPP     11
#  H    H   K   K   G         R    R   O    O   U    U   P    P   1 1
#  HHHHHH   KKKK    G  GGGG   R    R   O    O   U    U   P    P     1
#  H    H   K  K    G     G   RRRRR    O    O   U    U   PPPPP      1
#  H    H   K   K   G     G   R   R    O    O   U    U   P          1
#  H    H   K    K   GGGGG    R    R    OOOO     UUUU    P        11111
#
#-----------------------------------------------------------------------------
# 1.003 28 Mar 00 dbb Since v16819 & v16817 are currently installed in G/C,
#                     hks must be sure to upload to upper only after a
#                     partial version has move the rx/tx buffers.
#                     This hks does some smart uploading...
# 1.005  7 Apr 00 dbb Added HV tests, Reset Tests, Verbose switch
# 1.006 10 Apr 00 dbb Fixed a bug in cmd, reorganized hks tool into a
#                     collection of .pm files.
# 1.009 14 Apr 00 dbb Updated data structures to use test12
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# The 'use strict' direrctive forces the coder to follow rules that might keep
# him out of trouble by calling attention to undeclared variables and the like.
use strict;
use diagnostics -verbose;

#-----------------------------------------------------------------------------
#-----------------------------------------------------------------------------
sub group01tests {
  TestHeader("Test  1 -- Commanded Settings");
  useversion;
  removediags; # clear list so we can note updates
  my $i; my $j; my $k; # local variables

  # This is the complete list of housekeeping (hkp) items read into memitems
  # after every command (at least).  We use this list to build a list of memitems
  # we want to detect when changed.  A list of items to ignore is used to cull
  # the list used in the test, which not only verifies that the command has the
  # intended talkback, but also reports other housekeeping items affected.  The
  # ignore-list specifies hkp items we really wouldn't care if they changed during   # the test.

  my @CommandsToSend = (  0xc4, 0xc3, 0xc2, 0xc1 );
  my @PlacesToLook = (  "0x16f2", "0x16f4", "0x172a", "0x172c", "0x1712", "0x16e4"  );
  my @MasksToTry = (0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080,
                 0x0100, 0x0200, 0x0400, 0x0800, 0x1000, 0x2000, 0x4000, 0x8000  );
  my @TestSet;

    # The set below was taken from the ICD.  These are commands that take
    #  one parameter, which is readback at the specified hkp address, using
    #  the specified mask.  If the mask is FF, we assume the value read must
    #  equal the value commanded; otherwise we expect the mask bits to be 
    #  set when the value commanded is nonzero, reset otherwise.
  my $Tlimit = 255;
  my @CosTestSet = (   
     [ 0xc4, "0x16f2", 0x0008, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # LFRAXPWR
     [ 0x9c, "0x172c", 0x8000, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # MOTOR_SAFETY_OVERRIDE
     [ 0xc1, "0x172c", 0x0200, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # ACT_A
     [ 0xc2, "0x172c", 0x0400, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # ACT_B
     [ 0xc3, "0x172c", 0x1000, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # ACT_ENA
     [ 0xc4, "0x172c", 0x0100, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # LFRAXPWR
     [ 0xc3, "0x16f2", 0x0008, 0, 1, $Tlimit, \%hkinfo::HKPbasic ]); #  always bad?

  my @GalexTestSet = ( 
     [ 0xe1, "0x16f2", 0x0008, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # DGBOFFX
     [ 0xce, "0x1774", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # DHRAMPT
     [ 0x8d, "0x1726", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # HVMAX
     [ 0xb4, "0x170e", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # PCNT
     [ 0xb6, "0x1720", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # HVLO
     [ 0xba, "0x171c", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # PINT
     [ 0xbc, "0x1722", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # TOFF
     [ 0xc0, "0x172a", 0x0008, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # HVPWR
     [ 0xd1, "0x16f2", 0x0040, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # HVENA
     [ 0xc6, "0x16e2", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # DHVNM
     [ 0xcb, "0x16e2", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ],  # HVDAC
     [ 0xb6, "0x1720", 0x00FF, 0, 1, $Tlimit, \%hkinfo::HKPbasic ]); # DHVLOW

  # Seems these tests could cause a hang?  dbb 3-23-00
  # if ($main::project eq "COS") { return; } # safety abort!

  @TestSet = ($main::project eq "COS")?@CosTestSet:@GalexTestSet;
  ProcessTestSet(\@TestSet);
} # sub group01tests 

#-----------------------------------------------------------------------------
use hkgroup4;

#-----------------------------------------------------------------------------
sub group08tests {
  TestHeader("Test 8 -- Diagnostic Codes");
  useversion;
  my %group04set = (
    "softreset", 0x1C,
    "hvon",      0x08,
    "clrdiag",   0x00,
    "clrdiag",   0x00
  );
  system("clrdiag");

  my $pmask; my $tcommand; my $setting; my $eresu; my $tbadj; my $result; # declare local variables
  foreach $tcommand (keys %group04set) {
    $eresu = $group04set{$tcommand};
    $pmask = 0x00FF;
    $tbadj = "0x1780"; # top of diagnostic stack
    $result = cmdtest10($tcommand, $tbadj, $pmask, $eresu);
    # if (!$result) { last; } # drop out of this test on any failure
  } # for each setting
} # sub group08tests

sub greycode {
  my $whitecode =(shift(@_));
  return $whitecode ^ int($whitecode /2);
} # sub greycode

#-----------------------------------------------------------------------------
sub group10tests {
  TestHeader("Test 16 -- Memory Monitors");
  useversion;
  # This is the memory monitor test.  We specify a memory monitor that we know will have certain results
  #  when working correctly.  The test fails when the readback value does not match the expected value.
  # This test, and esp the readback values may need adjusting for various FSW versions.
  my @mmTestSpecs = ( 
                   [ 0x0856, 2, 0x64 ],  # trial address, indicator, and expected result
                   [ 0x0757, 2, 0x9b ],
                   [ 0x0658, 2, 0xff ],
                   [ 0x0559, 2, 0x00 ],
                   [ 0x045a, 2, 0xaa ],
                   [ 0x0000, 1, 0x02 ],
                   [ 0x0001, 1, 0x04 ],
                   [ 0x0002, 1, 0x89 ],
                   [ 0x0000, 0, 0x02 ],
                   [ 0x0001, 0, 0xde ],
                   [ 0x0002, 0, 0x00 ] );

 my @tmptmptmp = (
                   [ 0x0002, 0, 0x00 ],
                   [ 0x005c, 2, 0x00 ],   # stack
                   [ 0x005d, 2, 0x00 ],   # stack
                   [ 0x005e, 2, 0x00 ],   # stack
                   [ 0x005f, 2, 0x00 ],   # stack
                   [ 0x0060, 2, 0x00 ],   # stack
                   [ 0x0061, 2, 0x00 ],   # stack
                   [ 0x0062, 2, 0x00 ],   # stack
                   [ 0x0063, 2, 0x00 ],   # stack
                   [ 0x0064, 2, 0x00 ],   # stack
                   [ 0x0065, 2, 0x00 ],   # stack
                   [ 0x0066, 2, 0x00 ],   # stack
                   [ 0x0067, 2, 0x00 ],   # stack
                   [ 0x0068, 2, 0x00 ],   # stack
                   [ 0x0069, 2, 0x00 ],   # stack
                   [ 0x006a, 2, 0x00 ],   # stack
                   [ 0x006b, 2, 0x00 ],   # stack
                   [ 0x006c, 2, 0x00 ],   # stack
                   [ 0x006d, 2, 0x00 ],   # stack
                   [ 0x006e, 2, 0x00 ],   # stack
                   [ 0x006f, 2, 0x00 ],   # stack
                   [ 0x0070, 2, 0x00 ],   # stack
                   [ 0x0071, 2, 0x00 ],   # stack
                   [ 0x0072, 2, 0x00 ],   # stack
                   [ 0x0073, 2, 0x00 ],   # stack
                   [ 0x0074, 2, 0x00 ],   # stack
                   [ 0x0075, 2, 0x00 ],   # stack
                   [ 0x0076, 2, 0x00 ],   # stack
                   [ 0x0077, 2, 0x00 ],   # stack
                   [ 0x0078, 2, 0x00 ],   # stack
                   [ 0x0079, 2, 0x00 ],   # stack
                   [ 0x007a, 2, 0x00 ],   # stack
                   [ 0x007b, 2, 0x00 ],   # stack
                   [ 0x007c, 2, 0x00 ],   # stack
                   [ 0x007d, 2, 0x00 ],   # stack
                   [ 0x007e, 2, 0x00 ],   # stack
                   [ 0x007f, 2, 0x00 ],   # stack
                   [ 0x0080, 2, 0x00 ],   # stack
                   [ 0x0081, 2, 0x00 ],   # stack
                   [ 0x0082, 2, 0x00 ],   # stack
                   [ 0x0083, 2, 0x00 ],   # stack
                   [ 0x0084, 2, 0x00 ],   # stack
                   [ 0x0085, 2, 0x00 ],   # stack
                   [ 0x0086, 2, 0x00 ],   # stack
                   [ 0x0087, 2, 0x00 ],   # stack
                   [ 0x0088, 2, 0x00 ],   # stack
                   [ 0x0089, 2, 0x00 ],   # stack
                   [ 0x008a, 2, 0x00 ],   # stack
                   [ 0x008b, 2, 0x00 ],   # stack
                   [ 0x008c, 2, 0x00 ],   # stack
                   [ 0x008d, 2, 0x00 ],   # stack
                   [ 0x008e, 2, 0x00 ],   # stack
                   [ 0x008f, 2, 0x00 ],   # stack
                   [ 0x0090, 2, 0x00 ],   # stack
                   [ 0x0091, 2, 0x00 ],   # stack
                   [ 0x0092, 2, 0x00 ],   # stack
                   [ 0x0093, 2, 0x00 ],   # stack
                   [ 0x0094, 2, 0x00 ],   # stack
                   [ 0x0095, 2, 0x00 ],   # stack
                   [ 0x0096, 2, 0x00 ],   # stack
                   [ 0x0097, 2, 0x00 ],   # stack
                   [ 0x0098, 2, 0x00 ],   # stack
                   [ 0x0099, 2, 0x00 ],   # stack
                   [ 0x009a, 2, 0x00 ],   # stack
                   [ 0x009b, 2, 0x00 ],   # stack
                   [ 0x009c, 2, 0x00 ],   # stack
                   [ 0x009d, 2, 0x00 ],   # stack
                   [ 0x009e, 2, 0x00 ],   # stack
                   [ 0x009f, 2, 0x00 ],   # stack
                   [ 0x00a0, 2, 0x00 ],   # stack
                   [ 0x00a1, 2, 0x00 ],   # stack
                   [ 0x00a2, 2, 0x00 ],   # stack
                   [ 0x00a3, 2, 0x00 ],   # stack
                   [ 0x00a4, 2, 0x00 ],   # stack
                   [ 0x00a5, 2, 0x00 ],   # stack
                   [ 0x00a6, 2, 0x00 ],   # stack
                   [ 0x00a7, 2, 0x00 ],   # stack
                   [ 0x00a8, 2, 0x00 ],   # stack
                   [ 0x00a9, 2, 0x00 ],   # stack
                   [ 0x00aa, 2, 0x00 ],   # stack
                   [ 0x00ab, 2, 0x00 ],   # stack
                   [ 0x00ac, 2, 0x00 ],   # stack
                   [ 0x00ad, 2, 0x00 ],   # stack
                   [ 0x00ae, 2, 0x00 ],   # stack
                   [ 0x00af, 2, 0x00 ],   # stack
                   [ 0x00b0, 2, 0x00 ],   # stack
                   [ 0x00b1, 2, 0x00 ],   # stack
                   [ 0x00b2, 2, 0x00 ],   # stack
                   [ 0x00b3, 2, 0x00 ],   # stack
                   [ 0x00b4, 2, 0x00 ],   # stack
                   [ 0x00b5, 2, 0x00 ],   # stack
                   [ 0x00b6, 2, 0x00 ],   # stack
                   [ 0x00b7, 2, 0x00 ],   # stack
                   [ 0x00b8, 2, 0x00 ],   # stack
                   [ 0x00b9, 2, 0x00 ],   # stack
                   [ 0x00ba, 2, 0x00 ],   # stack
                   [ 0x00bb, 2, 0x00 ],   # stack
                   [ 0x00bc, 2, 0x00 ],   # stack
                   [ 0x00bd, 2, 0x00 ],   # stack
                   [ 0x00be, 2, 0x00 ],   # stack
                   [ 0x00bf, 2, 0x00 ],   # stack
                   [ 0x00c0, 2, 0x00 ],   # stack
                   [ 0x00c1, 2, 0x00 ],   # stack
                   [ 0x00c2, 2, 0x00 ],   # stack
                   [ 0x00c3, 2, 0x00 ],   # stack
                   [ 0x00c4, 2, 0x00 ],   # stack
                   [ 0x00c5, 2, 0x00 ],   # stack
                   [ 0x00c6, 2, 0x00 ],   # stack
                   [ 0x00c7, 2, 0x00 ],   # stack
                   [ 0x00c8, 2, 0x00 ],   # stack
                   [ 0x00c9, 2, 0x00 ],   # stack
                   [ 0x00ca, 2, 0x00 ],   # stack
                   [ 0x00cb, 2, 0x00 ],   # stack
                   [ 0x00cc, 2, 0x00 ],   # stack
                   [ 0x00cd, 2, 0x00 ],   # stack
                   [ 0x00ce, 2, 0x00 ],   # stack
                   [ 0x00cf, 2, 0x00 ],   # stack
                   [ 0x00d0, 2, 0x00 ],   # stack
                   [ 0x00d1, 2, 0x00 ],   # stack
                   [ 0x00d2, 2, 0x00 ],   # stack
                   [ 0x00d3, 2, 0x00 ],   # stack
                   [ 0x00d4, 2, 0x00 ],   # stack
                   [ 0x00d5, 2, 0x00 ],   # stack
                   [ 0x00d6, 2, 0x00 ],   # stack
                   [ 0x00d7, 2, 0x00 ],   # stack
                   [ 0x00d8, 2, 0x00 ],   # stack
                   [ 0x00d9, 2, 0x00 ],   # stack
                   [ 0x00da, 2, 0x00 ],   # stack
                   [ 0x00db, 2, 0x00 ],   # stack
                   [ 0x00dc, 2, 0x00 ],   # stack
                   [ 0x00dd, 2, 0x00 ],   # stack
                   [ 0x00de, 2, 0x00 ],   # stack
                   [ 0x00df, 2, 0x00 ],   # stack
                   [ 0x00e0, 2, 0x00 ],   # stack
                   [ 0x00e1, 2, 0x00 ],   # stack
                   [ 0x00e2, 2, 0x00 ],   # stack
                   [ 0x00e3, 2, 0x00 ],   # stack
                   [ 0x00e4, 2, 0x00 ],   # stack
                   [ 0x00e5, 2, 0x00 ],   # stack
                   [ 0x00e6, 2, 0x00 ],   # stack
                   [ 0x00e7, 2, 0x00 ],   # stack
                   [ 0x00e8, 2, 0x00 ],   # stack
                   [ 0x00e9, 2, 0x00 ],   # stack
                   [ 0x00ea, 2, 0x00 ],   # stack
                   [ 0x00eb, 2, 0x00 ],   # stack
                   [ 0x00ec, 2, 0x00 ],   # stack
                   [ 0x00ed, 2, 0x00 ],   # stack
                   [ 0x00ee, 2, 0x00 ],   # stack
                   [ 0x00ef, 2, 0x00 ],   # stack
                   [ 0x00f0, 2, 0x00 ],   # stack
                   [ 0x00f1, 2, 0x00 ],   # stack
                   [ 0x00f2, 2, 0x00 ],   # stack
                   [ 0x00f3, 2, 0x00 ],   # stack
                   [ 0x00f4, 2, 0x00 ],   # stack
                   [ 0x00f5, 2, 0x00 ],   # stack
                   [ 0x00f6, 2, 0x00 ],   # stack
                   [ 0x00f7, 2, 0x00 ],   # stack
                   [ 0x00f8, 2, 0x00 ],   # stack
                   [ 0x00f9, 2, 0x00 ],   # stack
                   [ 0x00fa, 2, 0x00 ],   # stack
                   [ 0x00fb, 2, 0x00 ],   # stack
                   [ 0x00fc, 2, 0x00 ],   # stack
                   [ 0x00fd, 2, 0x00 ],   # stack
                   [ 0x00fe, 2, 0x00 ],   # stack
                   [ 0x00ff, 2, 0x00 ],   # stack
                   [ 0x0002, 0, 0x00 ] );
   # This table tells converts index to housekeeping item name.
  my @resTable = ( "0x1738", "0x1739", "0x173a", "0x173b", "0x173c", "0x173d", "0x173e", "0x173f");
  my $index = 0;
  my $FailCntr = 0;
  my $mmTS;

  for $mmTS (0..$#mmTestSpecs) {
    my $greyidx = greycode($index);
    $main::TestCounter++;
    removediags; # clear list so we can note updates

    printf "Programming memory monitor $greyidx to %04X $mmTestSpecs[$mmTS][1]", $mmTestSpecs[$mmTS][0];
    sendcommand(0x8181,$greyidx,$mmTestSpecs[$mmTS][0],$mmTestSpecs[$mmTS][1]); # 
    printf " and checking result %02X ", $mmTestSpecs[$mmTS][2]; # 
    my %hsamples = split(" ", `hkprint 0x1738-173f`); # returns a list of byte values
    if (hex($hsamples{$resTable[$greyidx]}) == ($mmTestSpecs[$mmTS][2])) {
      print "Pass";
    } else {
      printf ":got $hsamples{$resTable[$greyidx]}:";
      print "*Fail*";
      $main::Failures++;  # tally one failure if any sub-test failed.
      $FailCntr++;
    } # else
    print "\n";
    if ($mmTestSpecs[$mmTS][1] == 2) { # then the point mon should agree
      sendcommand(0xacac, $mmTestSpecs[$mmTS][0], 0xaa, 0);
      if ((hex($main::memitems{"0x1734"}) & 0x00FF) != ($mmTestSpecs[$mmTS][2])) {
        system("hkprint 0x1734");
        print "pointmon does not agree!\n";
      } # if
    } # if
    $index++;
    if ($index > 7) { $index = 0 }; # modulo 8
  } # for 

  my @ShortTestSet =   ( [ 0xac, "0x1734", 0x00FF ]);  # DGBOFFX
  my @TestSet = @ShortTestSet;
  ProcessTestSet(\@TestSet);

  return $FailCntr; # non-zero means fail
  system("hkprint 0x1740-174e");
  system("hkprint 0x1738-173f");
  system("hkprint 0x1736");
} # sub group10tests

#-----------------------------------------------------------------------------
sub group20tests {
  TestHeader("Test 32 -- HV");
  my $pmask; my $tcommand; my $setting; my $eresu; my $tbadj; my $result; # declare local variables
  useversion;
# Testing has shown that powering off the PROMs is bad.  Let's not do it.
  removediags; # clear list so we can note updates
  cmdtest10("promon",  "0x172c", $pmask, 0x00);
  $tcommand = "cmd 0xc4c4 1 0 0";  # LFRAXPWR
  $tbadj    = "0x16f2";
  $pmask    = 0x0008;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0xc4c4 0 0 0";  # LFRAXPWR
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  removediags; # clear list so we can note updates
  $tcommand = "cmd 0xc4c4 1 0 0";  # LFRAXPWR
  $tbadj    = "0x172c";
  $pmask    = 0x0100;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0xc4c4 0 0 0";  # LFRAXPWR
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  removediags; # clear list so we can note updates
  $tcommand = "cmd 0x9c9c 1 0 0";  # MOTOR_SAFETY_OVER
  $pmask    = 0x8000;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0x9c9c 0 0 0";  # MOTOR_SAFETY_OVER
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  removediags; # clear list so we can note updates
  $tcommand = "cmd 0xc1c1 1 0 0";  # ACT_A
  $pmask    = 0x0200;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0xc1c1 0 0 0";  # ACT_A
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  removediags; # clear list so we can note updates
  $tcommand = "cmd 0xc2c2 1 0 0";  # ACT_B
  $pmask    = 0x0400;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0xc2c2 0 0 0";  # ACT_B
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  removediags; # clear list so we can note updates
  $tcommand = "cmd 0xc2c2 1 0 0";  # ACT_B
  $pmask    = 0x0400;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0xc2c2 0 0 0";  # ACT_B
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  removediags; # clear list so we can note updates
  $tcommand = "cmd 0xc2c2 1 0 0";  # ACT_B
  $pmask    = 0x0400;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0xc2c2 0 0 0";  # ACT_B
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  removediags; # clear list so we can note updates
  $tcommand = "cmd 0xc3c3 1 0 0";  # ACT_ENA
  $pmask    = 0x1000;
  $result   = cmdtest10($tcommand, $tbadj, $pmask, $pmask);
  $tcommand = "cmd 0xc3c3 0 0 0";  # ACT_ENA
  $result   = cmdtest10($tcommand, $tbadj, $pmask, 0);

  my $testing = 0;
  if ($testing) {
    $pmask = ($main::project eq "COS")?0xC0:0x84; # select the right mask bits
    cmdtest10("promoff", "0x172c", $pmask, $pmask);
    cmdtest10("promon",  "0x172c", $pmask, 0x00);
  }
} # sub group20tests

#-----------------------------------------------------------------------------
sub group40tests {
  TestHeader("Test 64 -- Watchdog");
  useversion;
  showversioninfo; # checks and sets watchdog
  commandedWDR;
} # sub group40tests

#-----------------------------------------------------------------------------
sub group80tests {
  my $pmask;
# my $tcommand; my $setting; my $eresu; my $tbadj; my $result; # declare local variables
  TestHeader("Test 128 -- Misc Commands");
  useversion;
  my $testing = 1;
  if ($testing) {
    $pmask = 0x4000;
    cmdtest10("wddisable",  "0x16e4", $pmask, 0x0000);
    cmdtest10("wdenable",  "0x16e4", $pmask, 0x4000);
    cmdtest10("clrdiag",  "0x1780", 0xFFFF, 0x0000); # DIAGC
    cmdtest10("hvilim 5",  "0x170a", 0xFF00, 0x0500); #  HV_IMAX
    cmdtest10("hvilim 0",  "0x170a", 0xFF00, 0x0000); #  HV_IMAX
    cmdtest10("hvilim 9",  "0x170a", 0xFF00, 0x0900); #  HV_IMAX
    cmdtest10("cmd dbdb 2a 0 0",  "0x16f2", 0x2000, 0x2000); #  MOTOR_ARM
    cmdtest10("cmd dcdc 05 0 0",  "0x170a", 0x00FF, 0x0005); #  DOOR_IMAX
    cmdtest10("cmd dcdc ff 0 0",  "0x170a", 0x00FF, 0x00FF); #  DOOR_IMAX
    cmdtest10("cmd dcdc 2a 0 0",  "0x170a", 0x00FF, 0x002A); #  DOOR_IMAX
    cmdtest10("cmd dbdb 0 0 0",  "0x16f2", 0x2000, 0x0000); #  MOTOR_ARM disarm
    cmdtest10("hvmax 7",  "0x1726", 0x00FF, 0x0007); #  HV_MAX A
    cmdtest10("hvmax 5",  "0x1726", 0x00FF, 0x0005); #  HV_MAX A
    cmdtest10("cmd 8d8d 0 0 0",  "0x1726", 0x00FF, 0x0000); #  HV_MAX A
    cmdtest10("hvlow 80",  "0x1720", 0x00FF, 0x0080); #  HV_LOW A
    cmdtest10("hvlow 07",  "0x1720", 0x00FF, 0x0007); #  HV_LOW A
    cmdtest10("cmd b6b6 8 0 0",  "0x1720", 0x00FF, 0x0008); #  HV_LOW A
    cmdtest10("cmd b6b6 a 0 0",  "0x1720", 0x00FF, 0x000a); #  HV_LOW A
    cmdtest10("cmd 9c9c 7 0 0",  "0x172c", 0x8000, 0x8000); #  MOTOR_SAFETY_OVER
    cmdtest10("cmd 9c9c 0 0 0",  "0x172c", 0x8000, 0x0000); #  MOTOR_SAFETY_OVER
  }
} # sub group80tests 

